@@ -1,4 +1,4 @@
-/* $Id: cpl_image_resample.c,v 1.56 2012/03/13 15:46:03 llundin Exp $
+/* $Id: cpl_image_resample.c,v 1.57 2012/06/13 13:27:56 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/03/13 15:46:03 $
- * $Revision: 1.56 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/06/13 13:27:56 $
+ * $Revision: 1.57 $
+ * $Name: cpl-6_2 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -97,6 +97,13 @@ inline static double cpl_image_get_interpolated_int(const int *,
                                                     double, double,
                                                     double *);
 
+/** @addtogroup cpl_image
+
+    Usage:  define the following preprocessor symbols as needed, 
+            then include this file
+*/
+/**@{*/
+
 /*-----------------------------------------------------------------------------
                             Function codes
  -----------------------------------------------------------------------------*/
@@ -1103,6 +1110,8 @@ cpl_error_code cpl_image_fill_jacobian(cpl_image *out,
 
 }
 
+/**@}*/
+
 #define CPL_CLASS CPL_CLASS_DOUBLE
 #undef CPL_OPERATION
 #include "cpl_image_resample_body.h"
