@@ -1,4 +1,4 @@
-/* $Id: cpl_image_stats.c,v 1.90 2011/08/17 16:39:00 cgarcia Exp $
+/* $Id: cpl_image_stats.c,v 1.94 2012/10/01 14:50:02 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: cgarcia $
- * $Date: 2011/08/17 16:39:00 $
- * $Revision: 1.90 $
- * $Name: cpl-6_1_1 $
+ * $Author: llundin $
+ * $Date: 2012/10/01 14:50:02 $
+ * $Revision: 1.94 $
+ * $Name: cpl-6_2 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -121,7 +121,7 @@
   Images can be CPL_TYPE_FLOAT, CPL_TYPE_INT or CPL_TYPE_DOUBLE.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */ 
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_min_window(const cpl_image * image,
@@ -146,7 +146,7 @@ double cpl_image_get_min_window(const cpl_image * image,
   Images can be CPL_TYPE_FLOAT, CPL_TYPE_INT or CPL_TYPE_DOUBLE.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */ 
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_min(const cpl_image * image)
@@ -280,7 +280,7 @@ double cpl_image_get_median_window(const cpl_image * image,
   Images can be CPL_TYPE_FLOAT, CPL_TYPE_INT or CPL_TYPE_DOUBLE.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */ 
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_median(const cpl_image * image)
@@ -525,7 +525,7 @@ double cpl_image_get_centroid_y_window(const cpl_image * image,
   Images can be CPL_TYPE_FLOAT, CPL_TYPE_INT or CPL_TYPE_DOUBLE.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
 */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_image_get_minpos_window(const cpl_image * image,
@@ -569,7 +569,7 @@ cpl_error_code cpl_image_get_minpos_window(const cpl_image * image,
   Images can be CPL_TYPE_FLOAT, CPL_TYPE_INT or CPL_TYPE_DOUBLE.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_image_get_minpos(const cpl_image * image,
@@ -677,31 +677,32 @@ cpl_error_code cpl_image_get_maxpos(const cpl_image * image,
 /*----------------------------------------------------------------------------*/
 /**
   @ingroup cpl_image
-  @brief    Computes first and second order image statistics using median.
-  @param    image   Input image.
-  @param    llx     Lower left x position (FITS convention, 1 for leftmost)
-  @param    lly     Lower left y position (FITS convention, 1 for lowest)
-  @param    urx     Upper right x position (FITS convention)
-  @param    ury     Upper right y position (FITS convention)
-  @param    sigma   Output computed sigma value.
-  @return   the median
-  @see      cpl_image_get_median_window()
+  @brief  Computes median and mean absolute median deviation on an image window
+  @param  image  Input image.
+  @param  llx    Lower left x position (FITS convention, 1 for leftmost)
+  @param  lly    Lower left y position (FITS convention, 1 for lowest)
+  @param  urx    Upper right x position (FITS convention)
+  @param  ury    Upper right y position (FITS convention)
+  @param  sigma  The mean of the absolute median deviation of the (good) pixels
+  @return The median of the non-bad pixels
+  @see    cpl_image_get_mad_window()
+
+  For each non-bad pixel in the window the absolute deviation from the median is
+  computed.
+  The mean of these absolute deviations in returned via sigma, while the median
+  itself is returned by the function.
+  The computed median and sigma may be a robust estimate of the mean and standard
+  deviation of the pixels. The sigma is however still sensitive to outliers.
+  See cpl_image_get_mad_window() for a more robust estimator.
 
-  This function takes an image in input. It tries to estimate the average
-  and standard deviation of the image by approximating them by resp. the
-  median and the average absolute distance to the median.
-  The median is the returned double. The average absolute distance to the
-  median is written into sigma.
   Images can be CPL_TYPE_FLOAT, CPL_TYPE_INT or CPL_TYPE_DOUBLE.
-  In case of error, the #_cpl_error_code_ code is set, and the returned double
-  is undefined.
+  On error the #_cpl_error_code_ code is set and the return value is undefined.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
-  - CPL_ERROR_ILLEGAL_INPUT if the image bad pixels map cannot be
-    retrieved
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
   - CPL_ERROR_INVALID_TYPE if the passed image type is not supported
-  - CPL_ERROR_DATA_NOT_FOUND if all the pixels are bad in image
+  - CPL_ERROR_ILLEGAL_INPUT if the specified window is illegal
+  - CPL_ERROR_DATA_NOT_FOUND if all the pixels are bad in the image window
  */
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_median_dev_window(const cpl_image * image,
@@ -735,27 +736,16 @@ double cpl_image_get_median_dev_window(const cpl_image * image,
 /*----------------------------------------------------------------------------*/
 /**
   @ingroup cpl_image
-  @brief    Computes first and second order image statistics using median.
-  @param    image   Input image.
-  @param    sigma   Output computed sigma value.
-  @return   the median
-  @see      cpl_image_get_median_window()
-
-  This function takes an image in input. It tries to estimate the average
-  and standard deviation of the image by approximating them by resp. the
-  median and the average absolute distance to the median.
-  The median is the returned double. The average absolute distance to the
-  median is written into sigma.
-  Images can be CPL_TYPE_FLOAT, CPL_TYPE_INT or CPL_TYPE_DOUBLE.
-  In case of error, the #_cpl_error_code_ code is set, and the returned double
-  is undefined.
+  @brief  Computes median and mean absolute median deviation on an image window
+  @param  image  Input image.
+  @param  sigma  The mean of the absolute median deviation of the (good) pixels
+  @return The median of the non-bad pixels
+  @see    cpl_image_get_median_dev_window()
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
-  - CPL_ERROR_ILLEGAL_INPUT if the image bad pixels map cannot be
-    retrieved
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
   - CPL_ERROR_INVALID_TYPE if the passed image type is not supported
-  - CPL_ERROR_DATA_NOT_FOUND if all the pixels are bad in image
+  - CPL_ERROR_DATA_NOT_FOUND if all the pixels are bad in the image
  */
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_median_dev(const cpl_image * image,
@@ -780,3 +770,103 @@ double cpl_image_get_median_dev(const cpl_image * image,
 
     return median;
 }
+
+/*----------------------------------------------------------------------------*/
+/**
+  @ingroup cpl_image
+  @brief  Computes median and median absolute deviation (MAD) on an image window
+  @param  image  Input image.
+  @param  llx    Lower left x position (FITS convention, 1 for leftmost)
+  @param  lly    Lower left y position (FITS convention, 1 for lowest)
+  @param  urx    Upper right x position (FITS convention)
+  @param  ury    Upper right y position (FITS convention)
+  @param  sigma  The median of the absolute median deviation of the good pixels
+  @return The median of the non-bad pixels
+  @see    cpl_image_get_median_window(), CPL_MATH_STD_MAD
+
+  For each non-bad pixel in the window the absolute deviation from the median is
+  computed.
+  The median of these absolute deviations in returned via sigma, while the median
+  itself is returned by the function.
+
+  If the pixels are gaussian, the computed sigma is a robust and consistent
+  estimate of the standard deviation in the sense that the standard deviation
+  is approximately k * MAD, where k is a constant equal to approximately 1.4826.
+  CPL defines CPL_MATH_STD_MAD as this scaling constant.
+
+  Images can be CPL_TYPE_FLOAT, CPL_TYPE_INT or CPL_TYPE_DOUBLE.
+  On error the #_cpl_error_code_ code is set and the return value is undefined.
+
+  Possible #_cpl_error_code_ set in this function:
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
+  - CPL_ERROR_INVALID_TYPE if the passed image type is not supported
+  - CPL_ERROR_ILLEGAL_INPUT if the specified window is illegal
+  - CPL_ERROR_DATA_NOT_FOUND if all the pixels are bad in the image window
+ */
+/*----------------------------------------------------------------------------*/
+double cpl_image_get_mad_window(const cpl_image * image,
+                                cpl_size          llx,
+                                cpl_size          lly,
+                                cpl_size          urx,
+                                cpl_size          ury,
+                                double          * sigma)
+{
+    cpl_stats * stats;
+    double median;
+
+
+    cpl_ensure_code(sigma != NULL, CPL_ERROR_NULL_INPUT);
+
+    stats = cpl_stats_new_from_image_window(image, CPL_STATS_MAD,
+                                            llx, lly,  urx, ury);
+
+    cpl_ensure(stats != NULL, cpl_error_get_code(), 0.0);               
+                                                                        
+    /* Should not be able to fail now */
+
+    median = cpl_stats_get_median(stats);
+    *sigma = cpl_stats_get_mad(stats);
+
+    cpl_stats_delete(stats);
+
+    return median;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @ingroup cpl_image
+  @brief  Computes median and median absolute deviation (MAD) on an image
+  @param  image  Input image.
+  @param  sigma  The median of the absolute median deviation of the good pixels
+  @return The median of the non-bad pixels
+  @see    cpl_image_get_mad_window()
+
+  Possible #_cpl_error_code_ set in this function:
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
+  - CPL_ERROR_INVALID_TYPE if the passed image type is not supported
+  - CPL_ERROR_DATA_NOT_FOUND if all the pixels are bad in the image
+ */
+/*----------------------------------------------------------------------------*/
+double cpl_image_get_mad(const cpl_image * image,
+                         double          * sigma)
+{
+    cpl_stats * stats;
+    double median;
+
+
+    cpl_ensure_code(sigma != NULL, CPL_ERROR_NULL_INPUT);
+
+    stats = cpl_stats_new_from_image(image, CPL_STATS_MAD);
+
+    cpl_ensure(stats != NULL, cpl_error_get_code(), 0.0);               
+                                                                        
+    /* Should not be able to fail now */
+
+    median = cpl_stats_get_median(stats);
+    *sigma = cpl_stats_get_mad(stats);
+
+    cpl_stats_delete(stats);
+
+    return median;
+}
+
