@@ -1,4 +1,4 @@
-/* $Id: cpl_imagelist_basic.c,v 1.55 2011/07/20 14:32:39 llundin Exp $
+/* $Id: cpl_imagelist_basic.c,v 1.57 2012/06/12 12:32:16 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/07/20 14:32:39 $
- * $Revision: 1.55 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/06/12 12:32:16 $
+ * $Revision: 1.57 $
+ * $Name: cpl-6_2 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -90,7 +90,7 @@
   list in2 is added to the image number n in the list in1.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
   - CPL_ERROR_ILLEGAL_INPUT if the input images have different sizes
  */
 /*----------------------------------------------------------------------------*/
@@ -177,7 +177,7 @@ cpl_error_code cpl_imagelist_divide(
   The passed image is added to each image of the passed image list.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_imagelist_add_image(
@@ -260,7 +260,7 @@ cpl_error_code cpl_imagelist_divide_image(
   @see      cpl_image_add_scalar()
  
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_imagelist_add_scalar(
@@ -341,7 +341,7 @@ cpl_error_code cpl_imagelist_multiply_scalar(
   @see      cpl_imagelist_add_scalar()
  
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_imagelist_divide_scalar(
@@ -370,7 +370,7 @@ cpl_error_code cpl_imagelist_divide_scalar(
   @see      cpl_image_logarithm()
  
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_imagelist_logarithm(
@@ -399,7 +399,7 @@ cpl_error_code cpl_imagelist_logarithm(
   @see      cpl_image_exponential()
  
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_imagelist_exponential(
@@ -428,7 +428,7 @@ cpl_error_code cpl_imagelist_exponential(
   @see      cpl_image_power()
  
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_imagelist_power(
@@ -459,7 +459,7 @@ cpl_error_code cpl_imagelist_power(
   The list may be partly modified if an error occurs.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_imagelist_normalise(
@@ -494,7 +494,7 @@ cpl_error_code cpl_imagelist_normalise(
   The input image list is modified.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
   - CPL_ERROR_INCOMPATIBLE_INPUT if lo_cut is bigger than hi_cut
  */
 /*----------------------------------------------------------------------------*/
@@ -531,7 +531,7 @@ cpl_error_code cpl_imagelist_threshold(
   The returned map has to be deallocated with cpl_image_delete().
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
   - CPL_ERROR_ILLEGAL_INPUT if the input image list is not valid
  */
 /*----------------------------------------------------------------------------*/
@@ -588,7 +588,7 @@ cpl_image * cpl_image_new_from_accepted(const cpl_imagelist * imlist)
   For integer pixel types, the averaging is performed using integer division.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
   - CPL_ERROR_ILLEGAL_INPUT if the input image list is not valid
  */
 /*----------------------------------------------------------------------------*/
@@ -661,8 +661,19 @@ cpl_image * cpl_imagelist_collapse_create(const cpl_imagelist * imlist)
   The input image list can be of type CPL_TYPE_INT, CPL_TYPE_FLOAT and
   CPL_TYPE_DOUBLE. The created image will be of the same type.
 
-  The bad pixel maps of the input frames are not taken into account, and 
-  the one of the created image is empty.
+  On success each pixel in the created image is the average of the non-rejected
+  values on the pixel position in the input image list.
+
+  For a given pixel position any bad pixels (i.e. values) are handled as
+  follows:
+  Given n bad values on a given pixel position, n/2 of those values are assumed
+  to be low outliers and n/2 of those values are assumed to be high outliers.
+  Any low or high rejection will first reject up to n/2 bad values and if more
+  values need to be rejected that rejection will take place on the good values.
+  This rationale behind this is to allow the rejection of outliers to include
+  bad pixels without introducing a bias.
+  If for a given pixel all values in the input image list are rejected, the
+  resulting pixel is set to zero and flagged as rejected.
 
   Possible #_cpl_error_code_ set in this function:
   - CPL_ERROR_NULL_INPUT if an the input pointer is NULL
@@ -678,19 +689,19 @@ cpl_image * cpl_imagelist_collapse_minmax_create(const cpl_imagelist * self,
 {
     const cpl_image * imgself = cpl_imagelist_get_const(self, 0);
     const cpl_type    type    = cpl_image_get_type(imgself);
-    const cpl_size    ni      = cpl_imagelist_get_size(self);
-    const cpl_size    nx      = cpl_image_get_size_x(imgself);
-    const cpl_size    ny      = cpl_image_get_size_y(imgself);
-    const cpl_size    nuse    = ni - nlow - nhigh;
+    const size_t      ni      = cpl_imagelist_get_size(self);
+    const size_t      nx      = cpl_image_get_size_x(imgself);
+    const size_t      ny      = cpl_image_get_size_y(imgself);
+    const size_t      nrej    = (size_t)(nlow + nhigh);
     cpl_image       * avg     = NULL;
 
     
-    cpl_ensure(self != NULL, CPL_ERROR_NULL_INPUT, NULL);
+    cpl_ensure(self != NULL,  CPL_ERROR_NULL_INPUT, NULL);
     cpl_ensure(cpl_imagelist_is_uniform(self)==0, CPL_ERROR_ILLEGAL_INPUT,
                NULL);
-    cpl_ensure(nlow  >= 0, CPL_ERROR_ILLEGAL_INPUT, NULL);
-    cpl_ensure(nhigh >= 0, CPL_ERROR_ILLEGAL_INPUT, NULL);
-    cpl_ensure(nuse  >  0, CPL_ERROR_ILLEGAL_INPUT, NULL);
+    cpl_ensure(nlow  >= 0,    CPL_ERROR_ILLEGAL_INPUT, NULL);
+    cpl_ensure(nhigh >= 0,    CPL_ERROR_ILLEGAL_INPUT, NULL);
+    cpl_ensure(ni    >  nrej, CPL_ERROR_ILLEGAL_INPUT, NULL);
     
     /* Switch on the data type */
     switch (type) {
@@ -862,45 +873,45 @@ cpl_imagelist_collapse_sigclip_create(const cpl_imagelist * self,
 /*----------------------------------------------------------------------------*/
 /**
   @ingroup cpl_imagelist
-  @brief    Compute the median of an image list to a single image
-  @param    imlist    the input images list
-  @return   the median image or NULL on error case.
-  @see cpl_imagelist_is_uniform()
-
-  The returned image has to be deallocated with cpl_image_delete().
-
-  The bad pixel maps of the input frames are not taken into account, and 
-  the one of the created image is empty.
+  @brief    Create a median image from the input imagelist
+  @param    self    The input image list
+  @return   The median image of the input pixel type or NULL on error
+  @note The created image has to be deallocated with cpl_image_delete()
 
   The input image list can be of type CPL_TYPE_INT, CPL_TYPE_FLOAT and
   CPL_TYPE_DOUBLE.
 
+  On success each pixel in the created image is the median of the values on
+  the same pixel position in the input image list. If for a given pixel all
+  values in the input image list are rejected, the resulting pixel is set to
+  zero and flagged as rejected.
+
+  The median is defined here as the middle value of an odd number of sorted
+  samples and for an even number of samples as the mean of the two central
+  values. Note that with an even number of samples the median may not be
+  among the input samples.
+
+  Also, note that in the case of an even number of integer data, the mean
+  value will be computed using integer arithmetic. Cast your integer data
+  to a floating point pixel type if that is not the desired behavior.
+
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
   - CPL_ERROR_ILLEGAL_INPUT if the input image list is not valid
-  - CPL_ERROR_INVALID_TYPE if the passed image list type is not supported
+  - CPL_ERROR_INVALID_TYPE if the passed image list pixel type is not supported
  */
 /*----------------------------------------------------------------------------*/
-cpl_image * cpl_imagelist_collapse_median_create(const cpl_imagelist * imlist)
+cpl_image * cpl_imagelist_collapse_median_create(const cpl_imagelist * self)
 {
-    cpl_image           *   median;
-    cpl_size                nx, ny;
-    cpl_type                type;
-    cpl_size                i, j, k;
+    cpl_image    * median = NULL;
+    const size_t   nx  = cpl_image_get_size_x(cpl_imagelist_get_const(self, 0));
+    const size_t   ny  = cpl_image_get_size_y(cpl_imagelist_get_const(self, 0));
+    const cpl_type type = cpl_image_get_type (cpl_imagelist_get_const(self, 0));
     
-    /* Check inputs */
-    cpl_ensure(imlist != NULL, CPL_ERROR_NULL_INPUT, NULL);
-    cpl_ensure(cpl_imagelist_is_uniform(imlist)==0, CPL_ERROR_ILLEGAL_INPUT,
+    cpl_ensure(self != NULL, CPL_ERROR_NULL_INPUT, NULL);
+    cpl_ensure(cpl_imagelist_is_uniform(self) == 0, CPL_ERROR_ILLEGAL_INPUT,
                NULL);
-    nx = cpl_image_get_size_x(imlist->images[0]);
-    ny = cpl_image_get_size_y(imlist->images[0]);
-    type = cpl_image_get_type(imlist->images[0]);
-
-    /* Create the output image */
-    median = cpl_image_new(nx, ny, type);
-
-    cpl_ensure(median, cpl_error_get_code(), NULL);
-
+    
     /* Switch on the data type */
     switch (type) {
 #define CPL_CLASS CPL_CLASS_DOUBLE
@@ -915,8 +926,6 @@ cpl_image * cpl_imagelist_collapse_median_create(const cpl_imagelist * imlist)
 #include "cpl_imagelist_basic_body.h"
 #undef CPL_CLASS
         default:
-            cpl_image_delete(median);
-            median = NULL;
             (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
     }
 
@@ -943,7 +952,7 @@ cpl_image * cpl_imagelist_collapse_median_create(const cpl_imagelist * imlist)
   The mode can be either CPL_SWAP_AXIS_XZ or CPL_SWAP_AXIS_YZ
   
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
   - CPL_ERROR_ILLEGAL_INPUT if mode is not equal to one of the possible
     values or if the image list is not valid
   - CPL_ERROR_INVALID_TYPE if the passed image list type is not supported
