@@ -1,4 +1,4 @@
-/* $Id: cpl_io_fits.c,v 1.73 2012/05/11 14:36:58 llundin Exp $
+/* $Id: cpl_io_fits.c,v 1.76 2012/10/10 15:00:48 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/05/11 14:36:58 $
- * $Revision: 1.73 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/10/10 15:00:48 $
+ * $Revision: 1.76 $
+ * $Name: cpl-6_2 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,9 +38,6 @@
 
 #include <string.h>
 
-/* Needed for patch of fits_flush_file(). */
-#include <fitsio2.h>
-
 #ifdef _OPENMP
 #include <omp.h>
 #else
@@ -150,7 +147,6 @@ static int cpl_io_fits_free(cpl_fitsfile_t *, int *)
     __attribute__((nonnull(2)))
 #endif
     ;
-static int cpl_fits_flush_file(fitsfile *, int *) CPL_ATTR_NONNULL;
 
 /*-----------------------------------------------------------------------------
                               Function definitions
@@ -426,6 +422,16 @@ int cpl_io_fits_open_diskfile(fitsfile **pfptr, const char * filename,
                           ": %d%s) (%d)", (const void*)*pfptr, filename,
                           iomode, iomode == rmiomode ? " for reading" : "",
                           (int)cpl_nfitsfiles);
+#endif
+#ifdef CPL_IO_FITS_REWIND
+	    /* A newly opened file points to the 1st HDU so do the same here */
+	    if (fits_movabs_hdu(*pfptr, 1, NULL, status)) {
+#ifdef CPL_IO_FITS_DEBUG
+                cpl_msg_debug(cpl_func, "Could not move to primary HDU: %s (%p) (I/O-"
+                              "mode: %d) (%d)", filename, (const void*)*pfptr,
+                              rmiomode, (int)cpl_nfitsfiles);
+#endif
+	    }
 #endif
             return *status;
         }
@@ -501,16 +507,15 @@ int cpl_io_fits_close_file(fitsfile *fptr, int *status)
         int          iomode;
         const char * name = cpl_io_fits_find_name(fptr, &iomode);
 
-        if (name != NULL) {
-            /* Must also flush read-only files */
+        if (name == NULL) {
+            /* This branch is used when CPL_IO_MODE is inactive */
+            (void)fits_close_file(fptr, status);
+        } else if (iomode != READONLY) {
 #ifdef CPL_IO_FITS_DEBUG
             cpl_msg_debug(cpl_func, "Flushing handle (%p) for: %s (%d) (%d)",
                           (const void*)fptr, name, iomode, (int)cpl_nfitsfiles);
 #endif
-            (void)cpl_fits_flush_file(fptr, status);
-        } else {
-            /* This branch is used when CPL_IO_MODE is inactive */
-            (void)fits_close_file(fptr, status);
+            (void)fits_flush_file(fptr, status);
         }
     }
 
@@ -837,19 +842,20 @@ static fitsfile * cpl_io_fits_reuse_fptr(const char * name, int iomode,
 
                 cpl_fitsfile = (cpl_fitsfile_t *)cx_list_get(cpl_fitslist, pos);
             }
-        }
 
-        if (cpl_fitsfile != NULL) {
-	    /* If we are reading, the file pointer is used by no one else */
-            /* If we are writing (i.e. called from within a cpl_*save(),
-               we may assume that no one else is currently using the file */
-            fptr = cpl_fitsfile->fptr;
-            if (iomode != READONLY) {
-                /* assert( cpl_fitsfile->iomode == READWRITE ); */
-                cpl_fitsfile->tid = omp_get_thread_num();
-	        cpl_fitsfile->writing = writing;
-	    }
-	}
+            /* Extend critical section, since it is cheap and just to be sure */
+            if (cpl_fitsfile != NULL) {
+                /* If we are reading, the file pointer is used by no one else */
+                /* If we are writing (i.e. called from within a cpl_*save(),
+                   we may assume that no one else is currently using the file */
+                fptr = cpl_fitsfile->fptr;
+                if (iomode != READONLY) {
+                    /* assert( cpl_fitsfile->iomode == READWRITE ); */
+                    cpl_fitsfile->tid = omp_get_thread_num();
+                    cpl_fitsfile->writing = writing;
+                }
+            }
+        }
     }
 #endif
 
@@ -900,42 +906,3 @@ static const char * cpl_io_fits_find_name(const fitsfile * fptr, int * piomode)
 
     return name;
 }
-
-
-/*----------------------------------------------------------------------------*/
-/**
-  @internal
-  @brief Perform proper flushing of CFITSIO files
-  @param  fptr    The CFITSIO pointer to flush
-  @param  status  Pointer to the CFITSIO error status
-  @return 0 iff successful
-  @see fits_flush_file() v. 3.28
-  @note The call to ffflsh() is done with clearbuf set to TRUE instead of FALSE
-
-*/
-/*----------------------------------------------------------------------------*/
-static int cpl_fits_flush_file(fitsfile *fptr,   /* I - FITS file pointer   */
-           int *status)      /* IO - error status                           */
-/*
-  Flush all the data in the current FITS file to disk. This ensures that if
-  the program subsequently dies, the disk FITS file will be closed correctly.
-*/
-{
-    int hdunum, hdutype;
-
-    if (*status > 0)
-        return(*status);
-
-    ffghdn(fptr, &hdunum);     /* get the current HDU number */
-
-    if (ffchdu(fptr,status) > 0)   /* close out the current HDU */
-        ffpmsg("ffflus could not close the current HDU.");
-
-    ffflsh(fptr, TRUE, status);  /* flush any modified IO buffers to disk */
-
-    if (ffgext(fptr, hdunum - 1, &hdutype, status) > 0) /* reopen HDU */
-        ffpmsg("ffflus could not reopen the current HDU.");
-
-    return(*status);
-}
-
