@@ -1,4 +1,4 @@
-/* $Id: cpl_math_const.h,v 1.9 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_math_const.h,v 1.10 2012/10/01 14:50:02 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.9 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/10/01 14:50:02 $
+ * $Revision: 1.10 $
+ * $Name: cpl-6_2 $
  */
 
 #ifndef CPL_MATH_CONST_H
@@ -257,6 +257,20 @@ CPL_BEGIN_DECLS
 /*----------------------------------------------------------------------------*/
 #define CPL_MATH_SIG_FWHM 0.4246609001440095213607514170514448098575705468921770
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief Standard deviation per Median Absolute Deviation for Gaussian data
+  @see cpl_image_get_mad_window()
+
+  For a Gaussian distribution the Median Absolute Deviation (MAD) is a robust
+  and consistent estimate of the Standard Deviation (STD) in the sense that the
+  STD is approximately K * MAD, where K is a constant equal to approximately
+  1.4826.
+
+ */
+/*----------------------------------------------------------------------------*/
+#define CPL_MATH_STD_MAD 1.4826
+
 /**@}*/
 
 CPL_END_DECLS
