@@ -1,4 +1,4 @@
-/* $Id: cpl_msg.c,v 1.54 2012/03/06 10:38:44 llundin Exp $
+/* $Id: cpl_msg.c,v 1.56 2012/10/29 09:52:08 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/03/06 10:38:44 $
- * $Revision: 1.54 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/10/29 09:52:08 $
+ * $Revision: 1.56 $
+ * $Name: cpl-6_2 $
  */
 
 
@@ -183,20 +183,6 @@
 #define INFO_STRING    "[ INFO  ] "
 #define DEBUG_STRING   "[ DEBUG ] "
 
-/* FIXME: Needed for deprecated cpl_msg_progress() */
-
-enum _cpl_clock_ {
-    CPL_CLOCK_RESET,
-    CPL_CLOCK_START,
-    CPL_CLOCK_STOP,
-    CPL_CLOCK_TIME
-};
-
-typedef enum _cpl_clock_ cpl_clock;
-
-static double cpl_tools_get_cputime(cpl_clock);
-
-
 inline static
 void cpl_msg_out(cpl_msg_severity, const char *, int,
                  const char *, va_list) CPL_ATTR_PRINTF(4,0);
@@ -891,7 +877,8 @@ cpl_error_code cpl_msg_set_log_level(cpl_msg_severity verbosity)
         char timeLabel[TIME_ISO8601_LENGTH];
 
         if ((logfile = fopen(logfile_name, "w")) == NULL)
-            return cpl_error_set_(CPL_ERROR_FILE_NOT_CREATED);
+            return cpl_error_set_message_(CPL_ERROR_FILE_NOT_CREATED, "%s",
+                                          logfile_name);
 
         (void)setvbuf(logfile, (char *) NULL, _IOLBF, 0);
 
@@ -1712,9 +1699,9 @@ void cpl_msg_progress(const char *component, int i, int iter,
     va_list     al;
     const char *c;
 
-    const  double tquiet = 10; /* Accept silence for this many seconds */
-    static double tstart = 0;  /* Initialize to avoid false warnings */
-    static double tend = 0;    /* Initialize to avoid false warnings */
+    const  double tquiet = 10.0; /* Accept silence for this many seconds */
+    static double tstart =  0.0; /* Initialize to avoid false warnings */
+    static double tend   =  0.0; /* Initialize to avoid false warnings */
     double tspent;
     static int iprev = 0;      /* Used to detect some illegal calls */
     static int nprev = 0;      /* Used to detect some illegal calls */
@@ -1742,9 +1729,7 @@ void cpl_msg_progress(const char *component, int i, int iter,
         iprev = 0;
         /* Assume caller printed a message before the loop started */
         /* Find out how much CPU was spent at this point */
-        tstart = cpl_tools_get_cputime(CPL_CLOCK_TIME);
-        /* Start counting the amount of CPU time spent */
-        (void)cpl_tools_get_cputime(CPL_CLOCK_START);
+        tstart = cpl_test_get_cputime();
         tend = 0;
         didmsg = 0;
         return;
@@ -1758,9 +1743,7 @@ void cpl_msg_progress(const char *component, int i, int iter,
     if (iter != nprev) return; /* cpl_ensure() ? */
 
     /* Compute the time spent in the loop so far */
-    (void)cpl_tools_get_cputime(CPL_CLOCK_STOP);
-    tspent = cpl_tools_get_cputime(CPL_CLOCK_TIME) - tstart;
-    if (i < iter-1) (void)cpl_tools_get_cputime(CPL_CLOCK_START);
+    tspent = cpl_test_get_cputime() - tstart;
 
     /* This fraction (rounded down) of iterations has been completed */
     percent = (i * 100) / iter;
@@ -1811,88 +1794,3 @@ void cpl_msg_progress(const char *component, int i, int iter,
 }
 
 /**@}*/
-
-/*----------------------------------------------------------------------------*/
-/**
-  @internal
-  @brief    Timer handling, including benchmarking.
-  @param    mode [CPL_CLOCK_START CPL_CLOCK_STOP CPL_CLOCK_TIME CPL_CLOCK_RESET]
-  @return   An elapsed time in seconds as a double.
-  @deprecated Use cpl_test_get_cputime() instead.
-
-  This routine is useful to benchmark the time spent in an image processing
-  routine. It is used in three steps:
-
-  1) Start the timer by calling @c cpl_tools_get_cputime(CPL_CLOCK_START)
-  2) Stop the timer by calling @c cpl_tools_get_cputime(CPL_CLOCK_STOP)
-  3) Get the total time by calling @c cpl_tools_get_cputime(CPL_CLOCK_TIME)
-
-  Steps 1) and 2) may be repeated pairwise more than once.
-  After any instance of step 2) step 3) may be repeated more than once.
-  cpl_tools_get_cputime(CPL_CLOCK_TIME) may also be called before step 1),
-  in which case it returns 0.
-
-  The call with CPL_CLOCK_START returns the total elapsed time in seconds since
-  the begining.
-  The call with CPL_CLOCK_STOP returns the time elapsed since the last
-  CPL_CLOCK_START call.
-  The call with CPL_CLOCK_TIME return the total time elapsed between all
-  previous CPL_CLOCK_START - CPL_CLOCK_STOP calls.
-
-  Additionally, the call with CPL_CLOCK_RESET sets the internal state of the
-  timing function to its initial state, something that may be necessary when
-  recovering from an error. In this case the function returns zero.
-
-  Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_ILLEGAL_INPUT
- */
-/*----------------------------------------------------------------------------*/
-static double cpl_tools_get_cputime(cpl_clock mode)
-{
-    /* The state of the static variables */
-    static cpl_clock pmode  = CPL_CLOCK_STOP;
-
-    /* clock() may wrap around after as little as 36 minutes,
-       so store the result in a double */
-    static clock_t chrono = 0;
-    static double  total  = 0.0;
-    clock_t        diff;
-    double         result;
-
-    switch (mode) {
-        case CPL_CLOCK_START:
-            cpl_ensure(pmode != CPL_CLOCK_START, CPL_ERROR_ILLEGAL_INPUT, 0);
-            chrono = clock();
-            result = chrono;
-            pmode = mode;
-            break;
-        case CPL_CLOCK_STOP:
-            cpl_ensure(pmode == CPL_CLOCK_START, CPL_ERROR_ILLEGAL_INPUT, 0);
-            /* At this point clock() may have wrapped around, causing it to be
-               much smaller than chrono. Subtraction must therefore be done in
-               type (clock_t). */
-            diff = clock() - chrono;
-            result = diff;
-            total += result;
-            pmode = mode;
-            break;
-        case CPL_CLOCK_TIME:
-            cpl_ensure(pmode != CPL_CLOCK_START, CPL_ERROR_ILLEGAL_INPUT, 0);
-            result = total;
-            break;
-        case CPL_CLOCK_RESET:
-            /* Set the static variables to their initial values */
-            pmode = CPL_CLOCK_STOP;
-            chrono = 0;
-            total = 0.0;
-            result = 0.0; /* Need to return something */
-            break;
-        default:
-            /* Only reached if cpl_clock is extended in error */
-            result = 0.0; /* Need to return something */
-            (void)cpl_error_set_(CPL_ERROR_UNSUPPORTED_MODE);
-            break;
-    }
-
-    return result / CLOCKS_PER_SEC;
-}
