@@ -1,4 +1,4 @@
-/* $Id: cpl_stats.c,v 1.56 2012/04/04 09:46:12 llundin Exp $
+/* $Id: cpl_stats.c,v 1.58 2012/07/30 23:39:10 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/04/04 09:46:12 $
- * $Revision: 1.56 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/07/30 23:39:10 $
+ * $Revision: 1.58 $
+ * $Name: cpl-6_2 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -62,6 +62,7 @@
 #define CPL_IMAGE_STATS_CENTROID        5
 #define CPL_IMAGE_STATS_MEDIAN          6
 #define CPL_IMAGE_STATS_MEDIAN_DEV      7
+#define CPL_IMAGE_STATS_MAD             8
 
 #define CONCAT(a,b) a ## _ ## b
 #define CONCAT2X(a,b) CONCAT(a,b)
@@ -111,6 +112,7 @@ struct _cpl_stats_
     double      mean;
     double      med;
     double      med_dev;
+    double      mad;
     double      stdev;
     double      flux;
     double      absflux;
@@ -202,19 +204,34 @@ double cpl_stats_get_median(const cpl_stats * in)
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    Get the median deviation from a cpl_stats object
-  @param    in  the cpl_stats object
-  @return   the median value
-  @see      cpl_stats_get_min()
+  @brief   Get the mean of the absolute median deviation from a cpl_stats object
+  @param   in  the cpl_stats object
+  @return  The mean of the absolute median deviation, or undefined on error
+  @see     cpl_stats_get_min()
  */
 /*----------------------------------------------------------------------------*/
 double cpl_stats_get_median_dev(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, 0.0);
-    cpl_ensure(in->mode & CPL_STATS_MEDIAN_DEV, CPL_ERROR_ILLEGAL_INPUT, 0);
+    cpl_ensure(in->mode & CPL_STATS_MEDIAN_DEV, CPL_ERROR_ILLEGAL_INPUT, 0.0);
     return in->med_dev;
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief   Get the median of the absolute median deviation
+  @param   in  the cpl_stats object
+  @return  The median of the absolute median deviation, or undefined on error
+  @see     cpl_stats_get_min()
+ */
+/*----------------------------------------------------------------------------*/
+double cpl_stats_get_mad(const cpl_stats * in)
+{
+    cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, 0.0);
+    cpl_ensure(in->mode & CPL_STATS_MAD, CPL_ERROR_ILLEGAL_INPUT, 0.0);
+    return in->mad;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Get the std. dev. from a cpl_stats object
@@ -436,6 +453,7 @@ void cpl_stats_delete(cpl_stats * stats)
   - CPL_STATS_MEAN
   - CPL_STATS_MEDIAN
   - CPL_STATS_MEDIAN_DEV
+  - CPL_STATS_MAD
   - CPL_STATS_STDEV
   - CPL_STATS_FLUX
   - CPL_STATS_ABSFLUX
@@ -538,8 +556,9 @@ cpl_stats * cpl_stats_new_from_image_window(
     cpl_ensure((mode & ~CPL_STATS_ALL) == 0, CPL_ERROR_UNSUPPORTED_MODE,
                NULL);
 
-    /* Need the median for the median deviation */
-    if (mode & CPL_STATS_MEDIAN_DEV) mode |= CPL_STATS_MEDIAN;
+    /* Need the median for the mean and median median absolute deviation */
+    if (mode & (CPL_STATS_MEDIAN_DEV | CPL_STATS_MAD))
+        mode |= CPL_STATS_MEDIAN;
 
     /* Get the bad pixels map */
     badmap = image->bpm == NULL ? NULL : cpl_mask_get_data_const(image->bpm);
@@ -716,6 +735,26 @@ cpl_stats * cpl_stats_new_from_image_window(
             }
         }
     }
+    if (mode & CPL_STATS_MAD) {
+        switch (image->type) {
+#define CPL_OPERATION CPL_IMAGE_STATS_MAD
+#define CPL_CLASS CPL_CLASS_DOUBLE
+#include "cpl_stats_body.h"
+#undef CPL_CLASS
+#define CPL_CLASS CPL_CLASS_FLOAT
+#include "cpl_stats_body.h"
+#undef CPL_CLASS
+#define CPL_CLASS CPL_CLASS_INT
+#include "cpl_stats_body.h"
+#undef CPL_CLASS
+#undef CPL_OPERATION
+        default:
+            /* See comment in previous switch() default: */
+            cpl_stats_delete(self);
+            (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+            return NULL;
+        }
+    }
 
     if (mode & CPL_STATS_MIN)  self->min  = min_pix;
     if (mode & CPL_STATS_MAX)  self->max  = max_pix;
@@ -793,7 +832,7 @@ cpl_stats * cpl_stats_new_from_image_window(
         self->max_y = 1 + max_pos / image->nx;
     }
 
-    if (mode &CPL_STATS_MEDIAN_DEV) self->med_dev = dev_sum / (double) npix;
+    if (mode & CPL_STATS_MEDIAN_DEV) self->med_dev = dev_sum / (double) npix;
 
     return self;
 }
@@ -841,7 +880,7 @@ cpl_stats * cpl_stats_new_from_image(const cpl_image * image,
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_stats_dump(const cpl_stats * self,
-                              cpl_stats_mode mode,
+                              cpl_stats_mode    mode,
                               FILE            * stream)
 {
 
@@ -858,6 +897,7 @@ cpl_error_code cpl_stats_dump(const cpl_stats * self,
     CPL_STATS_DUMP_ONE(CPL_STATS_MEAN,       mean,     "Mean",       ".8g");
     CPL_STATS_DUMP_ONE(CPL_STATS_MEDIAN,     med,      "Median",     ".8g");
     CPL_STATS_DUMP_ONE(CPL_STATS_MEDIAN_DEV, med_dev,  "Median dev", ".8g");
+    CPL_STATS_DUMP_ONE(CPL_STATS_MAD,        mad,      "MAD",        ".8g");
     CPL_STATS_DUMP_ONE(CPL_STATS_STDEV,      stdev,    "Std. dev",   ".8g");
     CPL_STATS_DUMP_ONE(CPL_STATS_FLUX,       flux,     "Flux",       ".8g");
     CPL_STATS_DUMP_ONE(CPL_STATS_ABSFLUX,    absflux,  "Abs flux",   ".8g");
