@@ -1,4 +1,4 @@
-/* $Id: cpl_stats.h,v 1.15 2011/07/20 09:40:05 llundin Exp $
+/* $Id: cpl_stats.h,v 1.17 2012/07/30 23:39:10 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/07/20 09:40:05 $
- * $Revision: 1.15 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/07/30 23:39:10 $
+ * $Revision: 1.17 $
+ * $Name: cpl-6_2 $
  */
 
 #ifndef CPL_STATS_H
@@ -114,15 +114,20 @@ enum _cpl_stats_mode_ {
      */
     CPL_STATS_CENTROID   =  1 << 11,
     /**
-     * The median deviation
+     * The mean of the absolute median deviation
      * @hideinitializer
      */
     CPL_STATS_MEDIAN_DEV =  1 << 12,
+    /**
+     * The median of the absolute median deviation
+     * @hideinitializer
+     */
+    CPL_STATS_MAD =  1 << 13,
     /**
      * All of the above
      * @hideinitializer
      */
-    CPL_STATS_ALL        = (1 << 13)-2
+    CPL_STATS_ALL        = (1 << 14)-2
 };
 
 
@@ -150,6 +155,7 @@ double cpl_stats_get_max(const cpl_stats *);
 double cpl_stats_get_mean(const cpl_stats *);
 double cpl_stats_get_median(const cpl_stats *);
 double cpl_stats_get_median_dev(const cpl_stats *);
+double cpl_stats_get_mad(const cpl_stats *);
 double cpl_stats_get_stdev(const cpl_stats *);
 double cpl_stats_get_flux(const cpl_stats *);
 double cpl_stats_get_absflux(const cpl_stats *);
