@@ -1,4 +1,4 @@
-/* $Id: cpl_test.h,v 1.60 2011/07/20 13:40:49 llundin Exp $
+/* $Id: cpl_test.h,v 1.61 2012/08/23 12:52:49 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/07/20 13:40:49 $
- * $Revision: 1.60 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/08/23 12:52:49 $
+ * $Revision: 1.61 $
+ * $Name: cpl-6_2 $
  */
 
 #ifndef CPL_TEST_H
@@ -1162,6 +1162,9 @@ size_t cpl_test_get_bytes_imagelist(const cpl_imagelist *) CPL_ATTR_PURE;
 
 cpl_flops cpl_test_get_flops(void) CPL_ATTR_PURE;
 
+cpl_size cpl_test_get_tested(void) CPL_ATTR_PURE;
+cpl_size cpl_test_get_failed(void) CPL_ATTR_PURE;
+
 CPL_END_DECLS
 
 #endif
