@@ -1,4 +1,4 @@
-/* $Id: cpl_tools_body.h,v 1.27 2011/08/17 16:48:55 cgarcia Exp $
+/* $Id: cpl_tools_body.h,v 1.28 2012/08/23 15:17:43 cgarcia Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: cgarcia $
- * $Date: 2011/08/17 16:48:55 $
- * $Revision: 1.27 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/08/23 15:17:43 $
+ * $Revision: 1.28 $
+ * $Name: cpl-6_2 $
  */
 
 #undef ADDTYPE
@@ -116,7 +116,7 @@ CPL_TYPE ADDTYPE(cpl_tools_get_median)(CPL_TYPE * self, cpl_size n)
         else      //Even number
         {
             (void)ADDTYPE(cpl_tools_quickselection)(self, n, n/2-1);
-            (void)ADDTYPE(cpl_tools_quickselection)(self + n/2, n - n/2-1, 0);
+            (void)ADDTYPE(cpl_tools_quickselection)(self + n/2, n - n/2, 0);
         }
     }
 
