@@ -1,4 +1,4 @@
-/* $Id: cpl_vector_fit_impl.h,v 1.12 2011/07/29 12:05:21 llundin Exp $
+/* $Id: cpl_vector_fit_impl.h,v 1.13 2012/06/13 12:51:53 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/07/29 12:05:21 $
- * $Revision: 1.12 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/06/13 12:51:53 $
+ * $Revision: 1.13 $
+ * $Name: cpl-6_2 $
  */
 
 #ifndef CPL_VECTOR_FIT_IMPL_H
@@ -285,9 +285,7 @@ get_chisq(cpl_size N, cpl_size D,
         const double *x_i = &(x[0+i*D]);
 
         /* Evaluate */
-        cpl_ensure( f(x_i,
-              a,
-              &fx_i) == 0, CPL_ERROR_ILLEGAL_INPUT, -1.0);
+        cpl_ensure( f(x_i, a, &fx_i) == 0, CPL_ERROR_ILLEGAL_INPUT, -1.0);
 
         /* Accumulate */
         if (sigma == NULL)
@@ -630,15 +628,18 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
         } while(cont);
 
         /* Get chi^2(a+da) */
-        if ((chi_sq_candidate = get_chisq(N, D, f, a_da,
-                          x_data, y_data, sigma_data)) < 0)
+        chi_sq_candidate = get_chisq(N, D, f, a_da, x_data, y_data, sigma_data);
+        /* Check for invalid candidate, including NaN */
+        if (chi_sq_candidate < 0.0 || chi_sq_candidate != chi_sq_candidate)
         {
             cpl_free(ia_local);
             cpl_matrix_delete(alpha);
             cpl_matrix_delete(beta);
             cpl_free(a_da);
             cpl_free(part);
-            return cpl_error_set_where_();
+            return cpl_error_get_code()
+                ? cpl_error_set_where_()
+                : cpl_error_set_(CPL_ERROR_CONTINUE);
         }
 
         if (chi_sq_candidate > chi_sq)
