@@ -1,4 +1,4 @@
-/* $Id: cpl_image_basic-test.c,v 1.94 2012/03/21 09:50:53 llundin Exp $
+/* $Id: cpl_image_basic-test.c,v 1.98 2012/11/07 09:46:07 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/03/21 09:50:53 $
- * $Revision: 1.94 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/11/07 09:46:07 $
+ * $Revision: 1.98 $
+ * $Name: cpl-6_2 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -67,6 +67,9 @@ static void cpl_image_flip_turn_test(const cpl_image *);
 static void cpl_image_turn_flip_test(void);
 static void cpl_image_copy_test(void);
 static void cpl_image_collapse_median_test(void);
+static void cpl_image_complex_mult(size_t o1, size_t o2, size_t n);
+static void cpl_image_complex_mult_d(size_t o1, size_t o2, size_t n);
+static void cpl_image_get_fwhm_test(void);
 
 /*-----------------------------------------------------------------------------
                                   Main
@@ -128,6 +131,7 @@ int main(void)
 
     /* Insert tests below */
 
+    cpl_image_get_fwhm_test();
     cpl_image_collapse_median_test();
     cpl_image_copy_test();
     cpl_image_turn_flip_test();
@@ -799,9 +803,9 @@ int main(void)
     cpl_test_nonnull( imf1 );
     cpl_test_nonnull( imi1 );
 
-    cpl_test_eq( cpl_image_get_flux(imd1), 2.5 );
-    cpl_test_eq( cpl_image_get_flux(imf1), 2.5 );
-    cpl_test_eq( cpl_image_get_flux(imi1), 2 );
+    cpl_test_abs( cpl_image_get_flux(imd1), 2.5, 0.0 );
+    cpl_test_abs( cpl_image_get_flux(imf1), 2.5, 0.0 );
+    cpl_test_abs( cpl_image_get_flux(imi1), 2.0, 0.0 );
     cpl_image_delete(imd1);
     cpl_image_delete(imf1);
     cpl_image_delete(imi1);
@@ -810,9 +814,9 @@ int main(void)
     cpl_test_zero( cpl_image_divide(imf, imtmp) );
     cpl_test_zero( cpl_image_divide(imi, imtmp) );
 
-    cpl_test_eq( cpl_image_get_flux(imd), 2.5 );
-    cpl_test_eq( cpl_image_get_flux(imf), 2.5 );
-    cpl_test_eq( cpl_image_get_flux(imi), 2 );
+    cpl_test_abs( cpl_image_get_flux(imd), 2.5, 0.0 );
+    cpl_test_abs( cpl_image_get_flux(imf), 2.5, 0.0 );
+    cpl_test_abs( cpl_image_get_flux(imi), 2.0, 0.0 );
 
     cpl_image_delete(imtmp);
     cpl_image_delete(imd);
@@ -860,7 +864,7 @@ int main(void)
     data3 = cpl_image_get_data_double_complex(imdc3);
 
     for (i = 0; i < IMAGESZ * IMAGESZ; i++) {
-    cpl_test_eq( data3[i], 3 * i + 3 * i * cpl_i );
+        cpl_test_abs( data3[i], 3 * i + 3 * i * cpl_i, 0.0 );
     }
 
     cpl_image_delete(imdc1);
@@ -885,13 +889,29 @@ int main(void)
     dataf3 = cpl_image_get_data_float_complex(imfc3);
 
     for (i = 0; i < IMAGESZ * IMAGESZ; i++) {
-    cpl_test_eq( dataf3[i], 3 * i + 3 * i * cpl_i );
+        cpl_test_abs( dataf3[i], 3 * i + 3 * i * cpl_i, 0.0 );
     }
 
     cpl_image_delete(imfc1);
     cpl_image_delete(imfc2);
     cpl_image_delete(imfc3);
 
+    /* test complex multiplication with several alignments and sizes */
+    cpl_image_complex_mult(0, 0, 64);
+    cpl_image_complex_mult(1, 1, 64);
+    cpl_image_complex_mult(1, 0, 64);
+    cpl_image_complex_mult(0, 0, 63);
+    cpl_image_complex_mult(1, 1, 63);
+    cpl_image_complex_mult(1, 0, 63);
+
+    /* test complex multiplication with several alignments and sizes */
+    cpl_image_complex_mult_d(0, 0, 64);
+    cpl_image_complex_mult_d(1, 1, 64);
+    cpl_image_complex_mult_d(1, 0, 64);
+    cpl_image_complex_mult_d(0, 0, 63);
+    cpl_image_complex_mult_d(1, 1, 63);
+    cpl_image_complex_mult_d(1, 0, 63);
+
     /* End of tests */
     return cpl_test_end(0);
 }
@@ -899,6 +919,201 @@ int main(void)
 /**@}*/
 
 
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    Test cpl_image_complex multiplication
+  @param    o1  image 1 offset
+  @param    o2  image 2 offset
+  @param    n   npixels
+  @return   void
+
+ */
+/*----------------------------------------------------------------------------*/
+
+static void cpl_image_complex_mult(size_t o1, size_t o2, size_t n)
+{
+    /* test multiplication, compares vector version with scalar version */
+    const size_t nl = n + 2;
+    const float complex cpl_i = csqrtf(-1.0);
+
+    float complex * dataf1_ = cpl_malloc(nl * nl * sizeof(float complex));
+    float complex * dataf2_ = cpl_malloc(nl * nl * sizeof(float complex));
+    float complex * dataf3;
+    float complex * dataf4 = cpl_malloc(n * n * sizeof(float complex));
+    float complex * dataf1 = dataf1_;
+    float complex * dataf2 = dataf2_;
+    size_t do1 = ((intptr_t)dataf1) % 16;
+    size_t do2 = ((intptr_t)dataf2) % 16;
+    cpl_image * imfc1;
+    cpl_image * imfc2;
+
+    cpl_msg_info(cpl_func, "Float complex vector multiply, "
+                 "size: %zu, offsets: %zu %zu", n * n, o1, o2);
+
+    do1 = (do1 ? (16u - do1) / sizeof(float complex) : 0u) + o1;
+    do2 = (do2 ? (16u - do2) / sizeof(float complex) : 0u) + o2;
+
+    dataf1 = dataf1 + do1;
+    dataf2 = dataf2 + do2;
+
+    for (size_t i = 0; i < n * n; i++) {
+        dataf1[i] = i + 12.2e-1 * i * cpl_i;
+        dataf2[i] = 3.23 * i + 2. * i * cpl_i;
+    }
+
+    /* add multiplication overflows, ignored in -ffast-math mode */
+    dataf1[0] = 1e30;
+    dataf2[0] = 1e30;
+    dataf1[1] = -1e30;
+    dataf2[1] = -1e30;
+    /* only one of the vector bad */
+    dataf2[2] = NAN + 1e30 * cpl_i;
+    dataf2[3] = -34.231e-5 + 4.111e+9 * cpl_i;
+    dataf2[4] = NAN + INFINITY * cpl_i;
+    dataf1[5] = 35.126e14 - INFINITY * cpl_i;
+    dataf2[5] = 35.126e14 - INFINITY * cpl_i;
+    /* subnormals */
+    dataf1[6] = 0.0000032e-37f - 0.0000001e-36f * cpl_i;
+    dataf2[6] = 0.16f - INFINITY * cpl_i;
+    dataf1[7] = 0.0000032e-37f - 0.0000001e-36f * cpl_i;
+    dataf2[7] = 1.16f - 0.0000001e-36f * cpl_i;
+
+    for (size_t i = 0; i < n * n; i++)
+        dataf4[i] = dataf1[i] * dataf2[i];
+
+    imfc1 = cpl_image_wrap(n, n, CPL_TYPE_FLOAT_COMPLEX, dataf1);
+    imfc2 = cpl_image_wrap(n, n, CPL_TYPE_FLOAT_COMPLEX, dataf2);
+
+    dataf3 = cpl_image_get_data_float_complex(imfc1);
+    /* multiply several times to get get multiplications
+     * involving special float values */
+    for (int j = 0; j < 3; j++) {
+        cpl_msg_info(cpl_func, "iteration: %d", j);
+        cpl_image_multiply(imfc1, imfc2);
+        for (size_t i = 0; i < n * n; i++) {
+            float r1 = creal(dataf3[i]);
+            float r2 = creal(dataf4[i]);
+            float i1 = cimag(dataf3[i]);
+            float i2 = cimag(dataf4[i]);
+
+            if (isnan(r1) || isnan(r2))
+                cpl_test_eq(isnan(r1), isnan(r2));
+            else if (isnan(i1) || isnan(i2))
+                cpl_test_eq(isnan(i1), isnan(i2));
+            else if (!isfinite(r1) || !isfinite(r2))
+                cpl_test_eq(isinf(r1), isinf(r2));
+            else if (!isfinite(i1) || !isfinite(i2))
+                cpl_test_eq(isinf(i1), isinf(i2));
+            else {
+                cpl_test_abs(i1, i2, FLT_EPSILON);
+                cpl_test_abs(r1, r2, FLT_EPSILON);
+            }
+
+            dataf4[i] *= dataf2[i];
+        }
+    }
+
+    cpl_image_unwrap(imfc1);
+    cpl_image_unwrap(imfc2);
+    cpl_free(dataf1_);
+    cpl_free(dataf2_);
+    cpl_free(dataf4);
+}
+
+
+static void cpl_image_complex_mult_d(size_t o1, size_t o2, size_t n)
+{
+    /* test multiplication, compares vector version with scalar version */
+    const size_t nl = n + 2;
+    const double complex cpl_i = csqrt(-1.0);
+
+    double complex * dataf1_ = cpl_malloc(nl * nl * sizeof(double complex));
+    double complex * dataf2_ = cpl_malloc(nl * nl * sizeof(double complex));
+    double complex * dataf3;
+    double complex * dataf4 = cpl_malloc(n * n * sizeof(double complex));
+    double complex * dataf1 = dataf1_;
+    double complex * dataf2 = dataf2_;
+    size_t do1 = ((intptr_t)dataf1) % 16;
+    size_t do2 = ((intptr_t)dataf2) % 16;
+    cpl_image * imfc1;
+    cpl_image * imfc2;
+
+    cpl_msg_info(cpl_func, "Double complex vector multiply, "
+                 "size: %zu, offsets: %zu %zu", n * n, o1, o2);
+
+    /* i386 aligns complex double 8 bytes */
+    do1 = (do1 ? (16u - do1) / sizeof(double) : 0u) + o1;
+    do2 = (do2 ? (16u - do2) / sizeof(double) : 0u) + o2;
+
+    dataf1 = (double complex *)((double *)dataf1 + do1);
+    dataf2 = (double complex *)((double *)dataf2 + do2);
+
+    for (size_t i = 0; i < n * n; i++) {
+        dataf1[i] = i + 12.2e-1 * i * cpl_i;
+        dataf2[i] = 3.23 * i + 2. * i * cpl_i;
+    }
+
+    /* add multiplication overflows, ignored in -ffast-math mode */
+    dataf1[0] = 1e30;
+    dataf2[0] = 1e30;
+    dataf1[1] = -1e30;
+    dataf2[1] = -1e30;
+    /* only one of the vector bad */
+    dataf2[2] = NAN + 1e30 * cpl_i;
+    dataf2[3] = -34.231e-5 + 4.111e+9 * cpl_i;
+    dataf2[4] = NAN + INFINITY * cpl_i;
+    dataf1[5] = 35.126e14 - INFINITY * cpl_i;
+    dataf2[5] = 35.126e14 - INFINITY * cpl_i;
+    /* subnormals */
+    dataf1[6] = 0.00000000032e-307 - 0.0000000001e-306 * cpl_i;
+    dataf2[6] = 0.16 - INFINITY * cpl_i;
+    dataf1[7] = 0.0000000032e-307 - 443.1 * cpl_i;
+    dataf2[7] = 1.16 - 0.00000000001e-306 * cpl_i;
+
+    for (size_t i = 0; i < n * n; i++)
+        dataf4[i] = dataf1[i] * dataf2[i];
+
+    imfc1 = cpl_image_wrap(n, n, CPL_TYPE_DOUBLE_COMPLEX, dataf1);
+    imfc2 = cpl_image_wrap(n, n, CPL_TYPE_DOUBLE_COMPLEX, dataf2);
+
+    dataf3 = cpl_image_get_data_double_complex(imfc1);
+    /* multiply several times to get get multiplications
+     * involving special double values */
+    for (int j = 0; j < 3; j++) {
+        cpl_msg_info(cpl_func, "iteration: %d", j);
+        cpl_image_multiply(imfc1, imfc2);
+        for (size_t i = 0; i < n * n; i++) {
+            double r1 = creal(dataf3[i]);
+            double r2 = creal(dataf4[i]);
+            double i1 = cimag(dataf3[i]);
+            double i2 = cimag(dataf4[i]);
+
+            if (isnan(r1) || isnan(r2))
+                cpl_test_eq(isnan(r1), isnan(r2));
+            else if (isnan(i1) || isnan(i2))
+                cpl_test_eq(isnan(i1), isnan(i2));
+            else if (!isfinite(r1) || !isfinite(r2))
+                cpl_test_eq(isinf(r1), isinf(r2));
+            else if (!isfinite(i1) || !isfinite(i2))
+                cpl_test_eq(isinf(i1), isinf(i2));
+            else {
+                cpl_test_abs(i1, i2, DBL_EPSILON);
+                cpl_test_abs(r1, r2, DBL_EPSILON);
+            }
+
+            dataf4[i] *= dataf2[i];
+        }
+    }
+
+    cpl_image_unwrap(imfc1);
+    cpl_image_unwrap(imfc2);
+    cpl_free(dataf1_);
+    cpl_free(dataf2_);
+    cpl_free(dataf4);
+}
+
+
 /*----------------------------------------------------------------------------*/
 /**
   @internal
@@ -950,10 +1165,10 @@ static void cpl_image_copy_test(void)
 
     cpl_test_zero(cpl_image_copy(im1, im2, 3, 1));
 
-    cpl_test_eq(cpl_image_get_flux(im1), 2 * IMAGESZ);
-    cpl_test_eq(cpl_image_get_flux_window(im1, 1, 1, 1, IMAGESZ), IMAGESZ);
-    cpl_test_eq(cpl_image_get_flux_window(im1, 2, 1, 2, IMAGESZ), IMAGESZ);
-    cpl_test_eq(cpl_image_get_flux_window(im1, 3, 1, IMAGESZ, IMAGESZ), 0);
+    cpl_test_abs(cpl_image_get_flux(im1), 2.0 * IMAGESZ, 0.0);
+    cpl_test_abs(cpl_image_get_flux_window(im1, 1, 1, 1, IMAGESZ), IMAGESZ, 0.0);
+    cpl_test_abs(cpl_image_get_flux_window(im1, 2, 1, 2, IMAGESZ), IMAGESZ, 0.0);
+    cpl_test_abs(cpl_image_get_flux_window(im1, 3, 1, IMAGESZ, IMAGESZ), 0.0, 0.0);
 
     /* Test copy of whole number of rows (all but two) */
     cpl_test_zero(cpl_image_copy(im1, im2, 1, 1));
@@ -962,20 +1177,17 @@ static void cpl_image_copy_test(void)
 
     cpl_test_zero(cpl_image_copy(im1, im2, 1, 3));
 
-    cpl_test_eq(cpl_image_get_flux(im1), 2 * IMAGESZ);
-    cpl_test_eq(cpl_image_get_flux_window(im1, 1, 1, IMAGESZ, 1), IMAGESZ);
-    cpl_test_eq(cpl_image_get_flux_window(im1, 1, 2, IMAGESZ, 2), IMAGESZ);
-    cpl_test_eq(cpl_image_get_flux_window(im1, 1, 3, IMAGESZ, IMAGESZ), 0);
+    cpl_test_abs(cpl_image_get_flux(im1), 2.0 * IMAGESZ, 0.0);
+    cpl_test_abs(cpl_image_get_flux_window(im1, 1, 1, IMAGESZ, 1), IMAGESZ, 0.0);
+    cpl_test_abs(cpl_image_get_flux_window(im1, 1, 2, IMAGESZ, 2), IMAGESZ, 0.0);
+    cpl_test_abs(cpl_image_get_flux_window(im1, 1, 3, IMAGESZ, IMAGESZ), 0.0, 0.0);
 
     cpl_image_delete(im1);
     cpl_image_delete(im2);
 
     return;
-
 }
 
-
-
 /*----------------------------------------------------------------------------*/
 /**
   @internal
@@ -1417,3 +1629,112 @@ static void cpl_image_flip_turn_test(const cpl_image * self)
 
     cpl_image_delete(tmp);
 }
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief  Test the CPL function
+  @return void
+  @see cpl_image_get_fwhm()
+
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_image_get_fwhm_test(void)
+{
+
+    /* Test case for DFS12346 */
+    const double x[] = { 
+        -13.9055,
+        -2.24811,
+        -4.60207,
+        12.0745,
+        -9.29113,
+        19.4861,
+        17.6999,
+        32.2417,
+        68.0725,
+        120.824,
+        172.361,
+        242.671,
+        302.048,
+        355.937,
+        348.458,
+        309.819,
+        240.489,
+        179.107,
+        117.223,
+        66.3061,
+        32.7849,
+        22.9367,
+    };
+
+    const cpl_size sz = (cpl_size)(sizeof(x)/sizeof(x[0]));   
+
+    /* x will _not_ be modified */
+    cpl_image * im1d = cpl_image_wrap_double(sz, 1, (double*)x);
+
+    cpl_size xmaxpos, ymaxpos;
+
+    double xfwhm, yfwhm, xref;
+    cpl_error_code code;
+
+    cpl_msg_info(cpl_func, "Testing cpl_image_get_fwhm() with %u elements",
+                 (unsigned)sz);
+
+    code = cpl_image_get_maxpos(im1d, &xmaxpos, &ymaxpos);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_eq(xmaxpos, 14);
+    cpl_test_eq(ymaxpos, 1);
+
+    /* Test some error input */
+    code = cpl_image_get_fwhm(NULL, xmaxpos, ymaxpos, &xfwhm, &yfwhm);
+    cpl_test_eq_error(code, CPL_ERROR_NULL_INPUT);
+
+    code = cpl_image_get_fwhm(im1d, xmaxpos, ymaxpos, NULL, &yfwhm);
+    cpl_test_eq_error(code, CPL_ERROR_NULL_INPUT);
+
+    code = cpl_image_get_fwhm(im1d, xmaxpos, ymaxpos, &xfwhm, NULL);
+    cpl_test_eq_error(code, CPL_ERROR_NULL_INPUT);
+
+    code = cpl_image_get_fwhm(im1d, 0, ymaxpos, &xfwhm, &yfwhm);
+    cpl_test_eq_error(code, CPL_ERROR_ACCESS_OUT_OF_RANGE);
+
+    code = cpl_image_get_fwhm(im1d, xmaxpos, 0, &xfwhm, &yfwhm);
+    cpl_test_eq_error(code, CPL_ERROR_ACCESS_OUT_OF_RANGE);
+
+    code = cpl_image_get_fwhm(im1d, 0, ymaxpos, &xfwhm, &yfwhm);
+    cpl_test_eq_error(code, CPL_ERROR_ACCESS_OUT_OF_RANGE);
+
+    code = cpl_image_get_fwhm(im1d, sz+1, ymaxpos, &xfwhm, &yfwhm);
+    cpl_test_eq_error(code, CPL_ERROR_ACCESS_OUT_OF_RANGE);
+
+    code = cpl_image_get_fwhm(im1d, xmaxpos, 2, &xfwhm, &yfwhm);
+    cpl_test_eq_error(code, CPL_ERROR_ACCESS_OUT_OF_RANGE);
+
+    code = cpl_image_get_fwhm(im1d, ymaxpos, ymaxpos, &xfwhm, &yfwhm);
+    cpl_test_eq_error(code, CPL_ERROR_DATA_NOT_FOUND);
+
+    /* Test with valid input */
+    code = cpl_image_get_fwhm(im1d, xmaxpos, ymaxpos, &xfwhm, &yfwhm);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_abs(xfwhm, 6.7434, 1e-4);
+    cpl_test_abs(yfwhm, -1.0, 0.0);
+
+    /* Rotate the image 90 degrees (this does not modfy the pixel buffer) */
+    code = cpl_image_turn(im1d, -1);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    /* Call again, with x/y variables swapped */
+    xref = xfwhm;
+    code = cpl_image_get_fwhm(im1d, ymaxpos, xmaxpos, &yfwhm, &xfwhm);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    /* Expect exactly same result */
+    cpl_test_abs(xfwhm, xref, 0.0);
+    cpl_test_abs(yfwhm, -1.0, 0.0);
+
+    cpl_test_eq_ptr((const void*)x, cpl_image_unwrap(im1d));
+
+}
