@@ -1,4 +1,4 @@
-/* $Id: cpl_imagelist_basic-test.c,v 1.60 2012/03/13 15:03:37 llundin Exp $
+/* $Id: cpl_imagelist_basic-test.c,v 1.62 2012/06/12 07:40:57 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/03/13 15:03:37 $
- * $Revision: 1.60 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/06/12 07:40:57 $
+ * $Revision: 1.62 $
+ * $Name: cpl-6_2 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -63,7 +63,7 @@
                             Private Function prototypes
  -----------------------------------------------------------------------------*/
 
-static void cpl_imagelist_collapse_median_create_tests(cpl_size);
+static void cpl_imagelist_collapse_median_create_tests(cpl_size, cpl_boolean);
 static void cpl_imagelist_collapse_median_create_bench(cpl_size);
 static void cpl_imagelist_collapse_minmax_create_bench(cpl_size);
 static
@@ -356,8 +356,10 @@ int main(void)
 
     cpl_imagelist_delete(imlist1);
 
-    cpl_imagelist_collapse_median_create_tests(NFRAMES);
-    cpl_imagelist_collapse_median_create_tests(NFRAMES*NFRAMES);
+    cpl_imagelist_collapse_median_create_tests(NFRAMES, CPL_TRUE);
+    cpl_imagelist_collapse_median_create_tests(NFRAMES, CPL_FALSE);
+    cpl_imagelist_collapse_median_create_tests(NFRAMES*NFRAMES, CPL_TRUE);
+    cpl_imagelist_collapse_median_create_tests(NFRAMES*NFRAMES, CPL_FALSE);
 
     if (do_bench) {
         /* cpl_msg_set_component_on(); */
@@ -490,48 +492,72 @@ int main(void)
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Unit test the CPL function
-  @param    mz  Number of planes to test
+  @param    mz       Number of planes to test
+  @param    test_bad True iff bad pixels are to be tested
   @return   void
 
  */
 /*----------------------------------------------------------------------------*/
-static void cpl_imagelist_collapse_median_create_tests(cpl_size mz)
+static void cpl_imagelist_collapse_median_create_tests(cpl_size mz,
+                                                       cpl_boolean test_bad)
 {
 
-    cpl_type       type = CPL_TYPE_DOUBLE;
+    const cpl_type pixel_type[] = {CPL_TYPE_INT, CPL_TYPE_FLOAT,
+                                   CPL_TYPE_DOUBLE};
+
     const cpl_size nx = IMAGESZ;
     const cpl_size ny = IMAGESZBIG;
-    const cpl_size nz = mz | 1; /* Test only odd sizes */
-    int            itype;
+    const cpl_size nz = mz | 1; /* Test only odd sizes + plus 1 rejected */
+    unsigned       itype;
 
 
     cpl_msg_info(cpl_func, "Testing with %" CPL_SIZE_FORMAT " %"
                  CPL_SIZE_FORMAT " X %" CPL_SIZE_FORMAT " images", nz, nx, ny);
 
-    for (itype = 0; itype < 2; itype++, type = CPL_TYPE_FLOAT) {
+    for (itype = 0;
+         itype < sizeof(pixel_type)/sizeof(pixel_type[0]); 
+         itype++) {
+
+        const cpl_type type = pixel_type[itype];
         cpl_imagelist * imglist = cpl_imagelist_new();
         cpl_image     * image;
         cpl_size        i;
+        cpl_error_code code;
 
         /* Fill imagelist with pixel-type type */
         /* The median of every pixel is zero */
         for (i = 0; i < nz; i++) {
+
             image = cpl_image_new(nx, ny, type);
             if (i > nz/2) {
-                cpl_test_zero(cpl_image_fill_noise_uniform(image, 1.0, (double)nz));
+                code = cpl_image_fill_noise_uniform(image, 1.0, (double)nz);
+                cpl_test_eq_error(code, CPL_ERROR_NONE);
             } else if (i > 0) {
-                cpl_test_zero(cpl_image_fill_noise_uniform(image, (double)-nz,
-                                                       -1.0));
+                code = cpl_image_fill_noise_uniform(image, (double)-nz, -1.0);
+                cpl_test_eq_error(code, CPL_ERROR_NONE);
             }
-            cpl_test_zero(cpl_imagelist_set(imglist, image, i));
+            code = cpl_imagelist_set(imglist, image, i);
+            cpl_test_eq_error(code, CPL_ERROR_NONE);
         }
 
         cpl_test_eq(cpl_imagelist_get_size(imglist), nz);
 
+        if (test_bad) {
+            /* Reject all pixels in the last, extra image */
+            image = cpl_image_duplicate(image);
+            code = cpl_mask_not(cpl_image_get_bpm(image));
+            cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+            code = cpl_imagelist_set(imglist, image, nz);
+            cpl_test_eq_error(code, CPL_ERROR_NONE);
+            cpl_test_eq(cpl_imagelist_get_size(imglist), nz + 1);
+        }
+
         image = cpl_imagelist_collapse_median_create(imglist);
 
-        cpl_test_nonnull( image);
-        cpl_test_error( CPL_ERROR_NONE);
+        cpl_test_error(CPL_ERROR_NONE);
+        cpl_test_nonnull(image);
+        cpl_test_eq(cpl_image_get_type(image), type);
 
         cpl_test_leq(cpl_image_get_min(image), 0.0);
         cpl_test_leq(cpl_image_get_max(image), 0.0);
