@@ -1,4 +1,4 @@
-/* $Id: cpl_imagelist_io-test.c,v 1.51 2011/08/17 14:44:45 llundin Exp $
+/* $Id: cpl_imagelist_io-test.c,v 1.52 2012/09/07 09:07:12 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/08/17 14:44:45 $
- * $Revision: 1.51 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/09/07 09:07:12 $
+ * $Revision: 1.52 $
+ * $Name: cpl-6_2 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -59,8 +59,14 @@ int main(void)
     cpl_vector    * eraser;
     cpl_size        i;
     FILE          * stream;
-    cpl_size        boxsize = IMAGE_SZ > 127 ? 5 :
-    ( IMAGE_SZ < 25 ? 1 : (IMAGE_SZ / 25) );
+#if IMAGE_SZ > 127
+    const cpl_size  boxsize = 5;
+#elif IMAGE_SZ < 25
+    /* FIXME: Will the tests pass with this ? */
+    const cpl_size  boxsize = 1;
+#else
+    const cpl_size  boxsize = IMAGE_SZ / 25;
+#endif
     cpl_error_code  error;
 
 
