@@ -1,4 +1,4 @@
-/* $Id: cpl_matrix-test.c,v 1.55 2011/12/13 14:42:11 llundin Exp $
+/* $Id: cpl_matrix-test.c,v 1.56 2012/09/07 09:02:29 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/12/13 14:42:11 $
- * $Revision: 1.55 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/09/07 09:02:29 $
+ * $Revision: 1.56 $
+ * $Name: cpl-6_2 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -187,7 +187,7 @@ int main(void)
     double     *dArray;
     double     *subdArray;
     double      value;
-    cpl_size    i, j, k, l;
+    cpl_size    i, j, k, l = 0; /* FIXME: What is l used for ? */
     cpl_size    r, c;
 
     cpl_matrix  *matrix;
