@@ -1,4 +1,4 @@
-/* $Id: cpl_stats-test.c,v 1.41 2012/02/15 16:20:35 llundin Exp $
+/* $Id: cpl_stats-test.c,v 1.44 2012/10/01 14:50:02 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/02/15 16:20:35 $
- * $Revision: 1.41 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/10/01 14:50:02 $
+ * $Revision: 1.44 $
+ * $Name: cpl-6_2 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -40,6 +40,7 @@
 #include "cpl_image_bpm.h"
 #include "cpl_memory.h"
 #include "cpl_test.h"
+#include "cpl_math_const.h"
 
 /*-----------------------------------------------------------------------------
                                    defines
@@ -148,6 +149,7 @@ int main(void)
     const cpl_type img_types[] = {CPL_TYPE_DOUBLE, CPL_TYPE_FLOAT, CPL_TYPE_INT};
     const cpl_stats_mode istats[] = {CPL_STATS_MIN,  CPL_STATS_MAX,
                                      CPL_STATS_MEDIAN, CPL_STATS_MEDIAN_DEV,
+                                     CPL_STATS_MAD,
                                      CPL_STATS_STDEV, CPL_STATS_FLUX,
                                      CPL_STATS_ABSFLUX, CPL_STATS_SQFLUX,
                                      CPL_STATS_MINPOS, CPL_STATS_MAXPOS,
@@ -259,6 +261,7 @@ int main(void)
         CPL_STAT_CMP(MEAN, mean);
         CPL_STAT_CMP(MEDIAN, median);
         CPL_STAT_CMP(MEDIAN_DEV, median_dev);
+        CPL_STAT_CMP(MAD, mad);
         CPL_STAT_CMP(STDEV, stdev);
         CPL_STAT_CMP(FLUX, flux);
         CPL_STAT_CMP(ABSFLUX, absflux);
@@ -282,6 +285,22 @@ int main(void)
         CPL_STAT_CMP_IMAGE(centroid_x);
         CPL_STAT_CMP_IMAGE(centroid_y);
 
+        cpl_test_abs( cpl_stats_get_median(statall),
+                      cpl_image_get_mad_window(img, 1, 1, IMAGE_SIZE_X,
+                                               IMAGE_SIZE_Y, &sig1),
+                      0.0);
+
+        cpl_test_abs( cpl_stats_get_median(statall),
+                      cpl_image_get_mad(img, &sig2), 0.0);
+
+        cpl_test_abs( sig1, sig2, 0.0);
+
+        /* Pixel values are not Gaussian, so the MAD cannot be used to estimate
+           the standard deviation. */
+        cpl_msg_info(cpl_func, "MAD as std.dev. estimator: MAD * 1.4826 = %g "
+                     "<=> %g = st.dev.", sig1 * CPL_MATH_STD_MAD,
+                     cpl_stats_get_stdev(statall));
+
         cpl_test_abs( cpl_stats_get_median(statall),
                       cpl_image_get_median_dev_window(img, 1, 1, IMAGE_SIZE_X,
                                                       IMAGE_SIZE_Y, &sig1),
