@@ -1,4 +1,4 @@
-/* $Id: cpl_test-test.c,v 1.31 2011/02/10 12:54:53 llundin Exp $
+/* $Id: cpl_test-test.c,v 1.32 2012/08/23 12:52:49 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/02/10 12:54:53 $
- * $Revision: 1.31 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/08/23 12:52:49 $
+ * $Revision: 1.32 $
+ * $Name: cpl-6_2 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -133,6 +133,7 @@ int main(void)
     cpl_errorstate   errstate;
     double           wallstart, cpustart;
     double           wallstop,  cpustop;
+    cpl_size         tested, failed;
     FILE           * fp;
     const unsigned int tsleep = 1;
     unsigned int       tleft;
@@ -142,6 +143,16 @@ int main(void)
     wallstart = cpl_test_get_walltime();
     cpustart  = cpl_test_get_cputime();
 
+    tested = cpl_test_get_tested();
+    failed = cpl_test_get_failed();
+    cpl_test_zero(tested);
+    cpl_test_zero(failed);
+
+    tested = cpl_test_get_tested();
+    failed = cpl_test_get_failed();
+    cpl_test_eq(tested, 2);
+    cpl_test_zero(failed);
+
     cpl_test(41);
     cpl_test_zero(0);
 
@@ -312,6 +323,9 @@ int main(void)
         /* FIXME: Create real, valid FITS here ? */
     }
 
+    failed = cpl_test_get_failed();
+    cpl_test_zero(failed);
+
     /* Turn off messaging during testing of expected failures */
 
     level = cpl_msg_get_level();
@@ -467,6 +481,9 @@ int main(void)
 
     cpl_msg_info(cpl_func, "Did %d failure tests", nxfail);
 
+    failed = cpl_test_get_failed();
+    cpl_test_eq(failed, nxfail);
+
     cpl_vector_delete(vec1);
     cpl_vector_delete(vec2);
     cpl_matrix_delete(mat1);
