@@ -1,4 +1,4 @@
-/* $Id: cpl_vector-test.c,v 1.94 2012/01/11 14:32:15 llundin Exp $
+/* $Id: cpl_vector-test.c,v 1.96 2012/08/27 11:35:40 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/01/11 14:32:15 $
- * $Revision: 1.94 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/08/27 11:35:40 $
+ * $Revision: 1.96 $
+ * $Name: cpl-6_2 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -117,6 +117,24 @@ int main(void)
     cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
     cpl_test_null(null);
 
+    /* Verify cpl_vector_get_median() with even number of samples (DFS12089) */
+    /* Test on unsorted vector */
+    tmp_vec = cpl_vector_new(4);
+    cpl_vector_set(tmp_vec, 0, 0.0);
+    cpl_vector_set(tmp_vec, 1, 1.0);
+    cpl_vector_set(tmp_vec, 2, 3.0);
+    cpl_vector_set(tmp_vec, 3, 2.0);
+    cpl_test_rel(1.5, cpl_vector_get_median(tmp_vec), DBL_EPSILON);
+    cpl_test_rel(1.5, cpl_vector_get_median_const(tmp_vec), DBL_EPSILON);
+    /* Test on sorted vector */
+    cpl_vector_set(tmp_vec, 0, 0.0);
+    cpl_vector_set(tmp_vec, 1, 1.0);
+    cpl_vector_set(tmp_vec, 2, 2.0);
+    cpl_vector_set(tmp_vec, 3, 3.0);
+    cpl_test_rel(1.5, cpl_vector_get_median(tmp_vec), DBL_EPSILON);
+    cpl_test_rel(1.5, cpl_vector_get_median_const(tmp_vec), DBL_EPSILON);
+    cpl_vector_delete(tmp_vec);
+    
     /* Create the vector sinus */
     cpl_test_nonnull( sinus = cpl_vector_new(VECTOR_SIZE) );
     
