@@ -1,4 +1,4 @@
-/* $Id: cpl_dfs.c,v 1.133 2012/05/11 13:27:12 llundin Exp $
+/* $Id: cpl_dfs.c,v 1.135 2012/08/17 13:53:40 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2012/05/11 13:27:12 $
- * $Revision: 1.133 $
- * $Name: cpl-6_1_1 $
+ * $Author: rpalsa $
+ * $Date: 2012/08/17 13:53:40 $
+ * $Revision: 1.135 $
+ * $Name: cpl-6_2 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1155,6 +1155,7 @@ static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset * self)
                                               "filename='%s'", filename);
                 }
 
+#ifndef CPL_IO_FITS_REWIND
                 /* The previous call may be reusing file handle opened for
                    previous I/O, so the file pointer needs to be moved */
                 if (fits_movabs_hdu(fproduct, 1, NULL, &error)) {
@@ -1162,6 +1163,7 @@ static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset * self)
                                               fits_movabs_hdu,
                                               "filename='%s'", filename);
                 }
+#endif
 
                 if (fits_read_card(fproduct, DATAMD5, card, &error)) {
                     code = cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, error,
@@ -1417,6 +1419,7 @@ static int cpl_is_fits(const char * self)
             (void)cpl_error_set_fits(CPL_ERROR_FILE_NOT_FOUND, error,
                                      fits_open_diskfile, "filename='%s'", self);
         error = 0;
+#ifndef CPL_IO_FITS_REWIND
     } else if (fits_movabs_hdu(fptr, 1, NULL, &error)) {
         /* The open call may be reusing file handle opened for
            previous I/O, so the file pointer needs to be moved */
@@ -1426,6 +1429,7 @@ static int cpl_is_fits(const char * self)
             (void)cpl_error_set_fits(CPL_ERROR_FILE_NOT_FOUND, error,
                                      fits_movabs_hdu, "filename='%s'", self);
         error = 0;
+#endif
     } else if (fits_read_imghdr(fptr, 0,  NULL, NULL, NULL, NULL, NULL, NULL,
                                 NULL, &error)) {
         is_fits = error == FILE_NOT_OPENED ? -2 : 0;
@@ -1587,10 +1591,11 @@ static FILE * cpl_dfs_paf_init(const char * instrume,
   will cause the printing of a line with QC.PART1.PART2.PART3 as key.
 
   Supported property-types:
-  CPL_TYPE_CHAR  (cast to cpl_size)
-  CPL_TYPE_INT   (cast to cpl_size)
-  CPL_TYPE_LONG  (cast to cpl_size)
-  CPL_TYPE_FLOAT (cast to double)
+  CPL_TYPE_CHAR       (cast to cpl_size)
+  CPL_TYPE_INT        (cast to cpl_size)
+  CPL_TYPE_LONG       (cast to cpl_size)
+  CPL_TYPE_LONG_LONG  (cast to cpl_size)
+  CPL_TYPE_FLOAT      (cast to double)
   CPL_TYPE_DOUBLE
   CPL_TYPE_STRING
  
@@ -1670,6 +1675,10 @@ cpl_error_code cpl_dfs_paf_dump(const cpl_propertylist * self, FILE * paf)
             err = cpl_dfs_paf_dump_int(usekey, cpl_property_get_long(prop),
                                        usecom, paf);
             break;
+        case CPL_TYPE_LONG_LONG:
+            err = cpl_dfs_paf_dump_int(usekey, cpl_property_get_long_long(prop),
+                                       usecom, paf);
+            break;
         case CPL_TYPE_FLOAT:
             err = cpl_dfs_paf_dump_double(usekey, cpl_property_get_float(prop),
                                          usecom, paf);
