@@ -1,4 +1,4 @@
-/* $Id: cpl_detector.c,v 1.32 2012/05/21 12:43:35 llundin Exp $
+/* $Id: cpl_detector.c,v 1.33 2012/10/29 14:27:53 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/05/21 12:43:35 $
- * $Revision: 1.32 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/10/29 14:27:53 $
+ * $Revision: 1.33 $
+ * $Name: cpl-6_2 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -149,7 +149,8 @@ static cpl_bivector * cpl_bivector_gen_ring_poisson(const double *,
   @param    noise       Output parameter: noise in the frame.
   @param    error       Output parameter: error on the noise.
   @return   the #_cpl_error_code_ or CPL_ERROR_NONE
-  @note     This function uses the pseudo-random number generator.
+  @see      rand()
+  @note     No calls to srand() are made from CPL
 
   This function is meant to compute the readout noise in a frame by means of a
   MonteCarlo approach. The input is a frame, usually a difference between two
@@ -199,8 +200,8 @@ cpl_error_code cpl_flux_get_noise_window(const cpl_image * diff,
   @param    bias        Output parameter: bias in the frame.
   @param    error       Output parameter: error on the bias.
   @return   the #_cpl_error_code_ or CPL_ERROR_NONE
-  @see      cpl_flux_get_noise_window()
-  @note     This function uses the pseudo-random number generator.
+  @see      cpl_flux_get_noise_window(), rand()
+  @note     No calls to srand() are made from CPL
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_flux_get_bias_window(const cpl_image * diff,
@@ -324,8 +325,8 @@ static cpl_error_code cpl_flux_get_window(
   @param    noise       Output parameter: noise in the frame.
   @param    error       Output parameter: error on the noise.
   @return   the #_cpl_error_code_ or CPL_ERROR_NONE
-  @see      cpl_flux_get_noise_window 
-  @note     This function uses the pseudo-random number generator.
+  @see      cpl_flux_get_noise_window(), rand()
+  @note     No calls to srand() are made from CPL
 
   The provided zone is an array of four integers specifying the zone to take
   into account for the computation. The integers specify a ring as x, y,
