@@ -1,4 +1,4 @@
-/* $Id: cpl_fit.c,v 1.42 2012/02/15 14:29:19 llundin Exp $
+/* $Id: cpl_fit.c,v 1.46 2012/06/15 14:26:56 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/02/15 14:29:19 $
- * $Revision: 1.42 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/06/15 14:26:56 $
+ * $Revision: 1.46 $
+ * $Name: cpl-6_2 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,6 +40,7 @@
 #include "cpl_polynomial_impl.h"
 #include "cpl_math_const.h"
 
+#include <errno.h>
 #include <math.h>
 /* Needed for memchr() */
 #include <string.h>
@@ -746,30 +747,24 @@ cpl_imagelist * cpl_fit_imagelist_polynomial(const cpl_vector    * x_pos,
 static int
 bigauss(const double x[], const double a[], double *result)
 {
+    errno = 0;
+
     if (a[5] == 0.0) {
-        if (x[0] == a[3]) {
-            *result = DBL_MAX;
-        }
-        else {
-            *result = 0.0; 
-        }
-    }
-    else if (a[6] == 0.0) { 
-        if (x[1] == a[4]) { 
-            *result = DBL_MAX;
-        }
-        else {
-            *result = 0.0;
-        }
-    }
-    else {
-        const double b1 = -0.5 / (1 - a[2] * a[2]);
+        *result = x[0] != a[3] ? 0.0 : DBL_MAX;
+    } else if (a[6] == 0.0) { 
+        *result = x[1] != a[4] ? 0.0 : DBL_MAX;
+    } else if (1.0 - a[2] * a[2] <= 0.0) { 
+        *result = a[1] != 0.0 ? DBL_MAX : 0.0;
+    } else {
+        const double b1 = -0.5 / (1.0 - a[2] * a[2]);
         const double b2 = (x[0] - a[3]) / a[5];
         const double b3 = (x[1] - a[4]) / a[6];
 
         *result = a[0]
-                + a[1] / (CPL_MATH_2PI * a[5] * a[6] * sqrt(1 - a[2] * a[2]))
+                + a[1] / (CPL_MATH_2PI * a[5] * a[6] * sqrt(1.0 - a[2] * a[2]))
                 * exp(b1 * (b2 * b2 - 2 * a[2] * b2 * b3 + b3 * b3));
+
+        if (errno) *result = 0.0;
     }
 
     return 0;
@@ -791,30 +786,22 @@ bigauss(const double x[], const double a[], double *result)
  * The i-th element of the returned @em result vector contains df/da[i].
  *
  * This function sets the result to zero if a5 or a6 (or both) are zero.
+ * This function sets the result to zero if a2 * a2 exceeds 1.
  */
 
 static int
 bigauss_derivative(const double x[], const double a[], double result[])
 {
 
-    if (a[5] == 0.0 || a[6] == 0.0) {
-
-        result[0] = 0.0;
-        result[1] = 0.0;
-        result[2] = 0.0;
-        result[3] = 0.0;
-        result[4] = 0.0;
-        result[5] = 0.0;
-        result[6] = 0.0;
-
-    } else {
+    errno = 0;
 
+    if (a[5] != 0.0 && a[6] != 0.0 && 1.0 - a[2] * a[2] >= 0.0) {
         const double b1 = 1 / (1 - a[2] * a[2]);
         const double b2 = (x[0] - a[3]) / a[5];
         const double b3 = (x[1] - a[4]) / a[6];
         const double b0 = b2 * b2 - 2 * a[2] * b2 * b3 + b3 * b3;
         const double b4 = exp(-0.5 * b1 * b0);
-        const double b5 = sqrt(1 - a[2] * a[2]);
+        const double b5 = sqrt(1.0 - a[2] * a[2]);
         const double b6 = CPL_MATH_PI * a[5] * a[6] * b5;
         const double b7 = 0.5 * a[1] * b4 / b6;
 
@@ -826,6 +813,15 @@ bigauss_derivative(const double x[], const double a[], double result[])
         result[5] =-b7 / a[5] * (b1 * b2 * (a[2] * b3 - b2) + 1);
         result[6] =-b7 / a[6] * (b1 * b3 * (a[2] * b2 - b3) + 1);
 
+        if (errno) {
+            result[0] = result[1] = result[2] = result[3]
+                = result[4] = result[5] = result[6] = 0.0;
+        }
+
+    } else {
+
+        result[0] = result[1] = result[2] = result[3]
+            = result[4] = result[5] = result[6] = 0.0;
     }
 
     return 0;
@@ -992,7 +988,20 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
                                       double          *angle,
                                       cpl_matrix     **phys_cov)
 {
-    cpl_size llx, lly, urx, ury;
+    const cpl_size mx = cpl_image_get_size_x(im);
+    const cpl_size my = cpl_image_get_size_y(im);
+    /* 
+     * Extraction box 
+     */
+
+    const cpl_size llx = CX_MAX(xpos - xsize/2, 1);
+    const cpl_size lly = CX_MAX(ypos - ysize/2, 1);
+    const cpl_size urx = CX_MIN(xpos + xsize/2, mx);
+    const cpl_size ury = CX_MIN(ypos + ysize/2, my);
+
+    const cpl_size nx = urx - llx + 1;
+    const cpl_size ny = ury - lly + 1;
+
     int     ia[] = {1, 1, 1, 1, 1, 1, 1};
     double  background;
     double  amplitude;
@@ -1001,12 +1010,14 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
     double  xcen, ycen;
     double  xsigma, ysigma;
     cpl_size ixcen, iycen;
-    cpl_size nx, ny, nrow;
-    int      i, j, row, bad;
-    int     found, invalid;
+    cpl_size nrow;
+    int      i, j, row;
+    int     invalid;
 
-    cpl_image     *box        = NULL;
-    cpl_image     *ebox       = NULL;
+    cpl_image     *ubox       = NULL;
+    const cpl_image *box      = NULL;
+    const cpl_image *ebox     = NULL;
+    cpl_image     *fbox       = NULL;
     cpl_vector    *a          = NULL;
     cpl_vector    *values     = NULL;
     cpl_vector    *dvalues    = NULL;
@@ -1034,12 +1045,12 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
         return cpl_error_set_message_(CPL_ERROR_NULL_INPUT,
                                      "Missing input data image.");
 
-    if (xpos  < 1                        ||
-        ypos  < 1                        ||
-        xpos  > cpl_image_get_size_x(im) ||
-        ypos  > cpl_image_get_size_y(im) ||
-        xsize > cpl_image_get_size_x(im) ||
-        ysize > cpl_image_get_size_y(im))
+    if (xpos  < 1  ||
+        ypos  < 1  ||
+        xpos  > mx ||
+        ypos  > my ||
+        xsize > mx ||
+        ysize > my)
         return cpl_error_set_message_(CPL_ERROR_ACCESS_OUT_OF_RANGE,
                                      "Fitting box extends beyond image.");
 
@@ -1048,8 +1059,8 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
                                      "Fitting box is too small.");
 
     if (im_err)
-        if (cpl_image_get_size_x(im_err) != cpl_image_get_size_x(im) || 
-            cpl_image_get_size_y(im_err) != cpl_image_get_size_y(im))
+        if (cpl_image_get_size_x(im_err) != mx || 
+            cpl_image_get_size_y(im_err) != my)
             return cpl_error_set_message_(CPL_ERROR_INCOMPATIBLE_INPUT,
                                      "Input images must have same size.");
 
@@ -1087,76 +1098,38 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
                                  cpl_type_get_name(cpl_image_get_type(im)));
     }
 
+    if (im_err && cpl_image_get_type(im) != cpl_image_get_type(im_err)) {
+        return cpl_error_set_message_(CPL_ERROR_TYPE_MISMATCH,
+                                      "Data and error images must have "
+                                      "same type, not %s and %s",
+                                      cpl_type_get_name(cpl_image_get_type(im)),
+                                      cpl_type_get_name(cpl_image_get_type
+                                                        (im_err)));
+    }
+
+    /* 
+     * Extract box and error box (if present)
+     */
+
+    if (llx == 1 && lly == 1 && urx == mx && ury == my) {
+        box = im;
+    } else {
+        box = ubox = cpl_image_extract(im, llx, lly, urx, ury);
+    }
+
     if (im_err) {
-        if (cpl_image_get_type(im) != cpl_image_get_type(im_err))
-            return cpl_error_set_message_(CPL_ERROR_TYPE_MISMATCH,
-                           "Data and error images must have same type");
-        switch(cpl_image_get_type(im_err)) { /* List of legal types */
-        case CPL_TYPE_INT:
-        case CPL_TYPE_FLOAT:
-        case CPL_TYPE_DOUBLE:
-            break;
-        default:
-            return cpl_error_set_message_(CPL_ERROR_INVALID_TYPE,
-                           "Cannot fit a gaussian to %s values",
-                           cpl_type_get_name(cpl_image_get_type(im_err)));
+        if (box == im) {
+            ebox = im_err;
+        } else {
+            ebox = fbox = cpl_image_extract(im_err, llx, lly, urx, ury);
         }
-    }
-
-    /* 
-     * Extraction box 
-     */
-
-    llx = xpos - xsize/2;
-    lly = ypos - ysize/2;
-    urx = xpos + xsize/2;
-    ury = ypos + ysize/2;
-
-    if (llx < 1)
-        llx = 1;
-    if (lly < 1)
-        lly = 1;
-    if (urx > cpl_image_get_size_x(im))
-        urx = cpl_image_get_size_x(im);
-    if (ury > cpl_image_get_size_y(im))
-        ury = cpl_image_get_size_y(im);
-
-
-    /* 
-     * Extract box and error box (if present), and convert all to FLOAT 
-     * if needed.
-     */
-
-    box = cpl_image_extract(im, llx, lly, urx, ury);
-
-    nx = cpl_image_get_size_x(box);
-    ny = cpl_image_get_size_y(box);
-
-/* *+
-    if (cpl_image_get_type(box) != CPL_TYPE_FLOAT) {
-        cpl_image *tmp_image = cpl_image_cast(box, CPL_TYPE_FLOAT);
-        cpl_image_delete(box);
-        box = tmp_image;
-    }
-+* */
-
-    if (im_err) {
-        ebox = cpl_image_extract(im_err, llx, lly, urx, ury);
 
         if (cpl_image_get_min(ebox) <= 0.0) {
-            cpl_image_delete(box);
-            cpl_image_delete(ebox);
+            cpl_image_delete(ubox);
+            cpl_image_delete(fbox);
             return cpl_error_set_message_(CPL_ERROR_ILLEGAL_INPUT,
                                "Non-positive errors found in error image.");
         }
-
-/* *+
-        if (cpl_image_get_type(ebox) != CPL_TYPE_FLOAT) {
-            cpl_image *tmp_image = cpl_image_cast(ebox, CPL_TYPE_FLOAT);
-            cpl_image_delete(ebox);
-            ebox = tmp_image;
-        }
-+* */
     }
 
 
@@ -1177,8 +1150,8 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
 
             cpl_array_get_double(parameters, i, &invalid);
             if (invalid) {
-                cpl_image_delete(box);
-                cpl_image_delete(ebox);
+                cpl_image_delete(ubox);
+                cpl_image_delete(fbox);
                 return cpl_error_set_message_(
                        CPL_ERROR_ILLEGAL_INPUT,
                        "Missing frozen value for parameter %d.", i);
@@ -1190,8 +1163,8 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
          */
 
         if (count == 7) {
-            cpl_image_delete(box);
-            cpl_image_delete(ebox);
+            cpl_image_delete(ubox);
+            cpl_image_delete(fbox);
             return cpl_error_set_message_(
                    CPL_ERROR_ILLEGAL_INPUT, "No free parameters");
         }
@@ -1228,8 +1201,8 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
          * position of max and sigmas...
          */
 
-        cpl_image_delete(box);
-        cpl_image_delete(ebox);
+        cpl_image_delete(ubox);
+        cpl_image_delete(fbox);
         cpl_array_set_double (parameters, 0, background);
         cpl_array_set_double (parameters, 1, 0.0);
         cpl_array_set_invalid(parameters, 2);
@@ -1258,8 +1231,7 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
     if (normalisation > 0.0) {
         cpl_image_get_maxpos(box, &ixcen, &iycen);
         amplitude = cpl_image_get_max(box) - background;
-    }
-    else {
+    } else {
         cpl_image_get_minpos(box, &ixcen, &iycen);
         amplitude = cpl_image_get_min(box) - background;
     }
@@ -1290,60 +1262,52 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
         ysigma = cpl_array_get_double(parameters, 6, &invalid);
         if (invalid) {
             int           xhalf = 0;
-            double        prev, next;
-            const double  value = amplitude / 2 + background;
+            const double  value = amplitude / 2.0 + background;
+            cpl_boolean   found = CPL_FALSE;
 
-//            data += nx * (iycen - 1);
-
-            found = 0;
             if (normalisation > 0.0) {
                 for (i = 1; i < nx; i++) {
+                    int bad;
+                    const double prev = cpl_image_get(box, i,     iycen, &bad);
+                    const double next = cpl_image_get(box, i + 1, iycen, &bad);
 
-                    prev = cpl_image_get(box, i, iycen, &bad);
-                    next = cpl_image_get(box, i + 1, iycen, &bad);
-
-                    if (prev < value && next >= value) {
+                    if (prev < value && value <= next) {
                         xhalf = i - 1;
-                        found = 1;
+                        found = CPL_TRUE;
                         break;
-                    }
-                    else if (prev > value && next <= value) {
+                    } else if (value < prev && next <= value) {
                         xhalf = i + 1;
-                        found = 1;
+                        found = CPL_TRUE;
                         break;
                     }
                 }
-            }
-            else {
+            } else {
                 for (i = 1; i < nx; i++) {
-                    prev = cpl_image_get(box, i, iycen, &bad);
-                    next = cpl_image_get(box, i + 1, iycen, &bad);
+                    int bad;
+                    const double prev = cpl_image_get(box, i,     iycen, &bad);
+                    const double next = cpl_image_get(box, i + 1, iycen, &bad);
 
-                    if (prev > value && next <= value) {
+                    if (value < prev && next <= value) {
                         xhalf = i - 1;
-                        found = 1;
+                        found = CPL_TRUE;
                         break;
-                    }
-                    else if (prev < value && next >= value) {
+                    } else if (prev < value && value <= next) {
                         xhalf = i + 1;
-                        found = 1;
+                        found = CPL_TRUE;
                         break;
                     }
                 }
             }
             if (found) {
                 xsigma = ysigma = fabs(xcen - xhalf);
-            }
-            else {
+            } else {
                 xsigma = (double)xsize;
                 ysigma = (double)ysize;
             }
-        }
-        else {
+        } else {
             xsigma = ysigma;
         }
-    }
-    else {
+    } else {
         ysigma = cpl_array_get_double(parameters, 6, &invalid);
         if (invalid) {
             ysigma = xsigma;
@@ -1410,8 +1374,8 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
          * Too few values for 7 free parameters!
          */
 
-        cpl_image_delete(box);
-        cpl_image_delete(ebox);
+        cpl_image_delete(ubox);
+        cpl_image_delete(fbox);
         cpl_vector_delete(a);
         return cpl_error_set_message_(CPL_ERROR_SINGULAR_MATRIX, "%"
                                       CPL_SIZE_FORMAT " < 7 free parameters.",
@@ -1426,23 +1390,25 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image *im,
     row = 0;
     for (j = 0; j < ny; j++) {
         for (i = 0; i < nx; i++) {
+            int bad;
             const double value = cpl_image_get(box, i+1, j+1, &bad);
-            if (bad)
-                continue;
-            cpl_matrix_set(positions, row, 0, i+1);
-            cpl_matrix_set(positions, row, 1, j+1);
-            cpl_vector_set(values, row, value);
-            if (ebox) {
-                const double evalue = cpl_image_get(ebox, i+1, j+1, &bad);
-                cpl_vector_set(dvalues, row, evalue);
+
+            if (!bad) {
+                cpl_matrix_set(positions, row, 0, i+1);
+                cpl_matrix_set(positions, row, 1, j+1);
+                cpl_vector_set(values, row, value);
+                if (ebox) {
+                    const double evalue = cpl_image_get(ebox, i+1, j+1, &bad);
+                    cpl_vector_set(dvalues, row, evalue);
+                }
+                if (!bad)
+                    row++;
             }
-            row++;
         }
     }
 
-    cpl_image_delete(box);
-    cpl_image_delete(ebox);
-
+    cpl_image_delete(ubox);
+    cpl_image_delete(fbox);
 
     /*
      * The "if" below may look absurd. But we need to compute
