@@ -1,4 +1,4 @@
-/* $Id: cpl_gasgano.c,v 1.21 2012/05/11 21:03:24 llundin Exp $
+/* $Id: cpl_gasgano.c,v 1.22 2012/05/25 10:02:51 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2006 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/05/11 21:03:24 $
- * $Revision: 1.21 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/05/25 10:02:51 $
+ * $Revision: 1.22 $
+ * $Name: cpl-6_2 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -56,6 +56,7 @@
 #include "cpl_dfs.h"
 #include "cpl_errorstate.h"
 #include "cpl_version.h"
+#include "cpl_memory.h"
 
 #include "org_eso_cpl_jni_CPLControl.h"
 #include "org_eso_cpl_jni_JNIParameterImp.h"
@@ -1276,9 +1277,6 @@ static void default_log_message_handler(const cxchar *domain,
    jstring jMsg;
    int ok = 1;
 
-    /* Keeps compiler quiet (Modified from cxmessages.c) */
-    ptr = 0;
-
    ok = ok &&
       (env = currentEnv);
    jMsg = NULL;
@@ -1297,7 +1295,10 @@ static void default_log_message_handler(const cxchar *domain,
                                    (jint) flags);
    }
    else {
-      fprintf(stderr, "Lost log message: %s\n", msg ? msg : "null");
+      /* Use ptr to avoid compiler warning */
+      char * nullmsg = cpl_sprintf("null (ptr=%p)", (const void*)ptr);
+      fprintf(stderr, "Lost log message: %s\n", msg ? msg : nullmsg);
+      cpl_free(nullmsg);
    }
 }
 
