@@ -1,4 +1,4 @@
-/* $Id: cpl_frameset_io-test.c,v 1.22 2011/12/13 12:58:16 llundin Exp $
+/* $Id: cpl_frameset_io-test.c,v 1.23 2012/07/30 18:05:38 llundin Exp $
  *
  *   This file is part of the ESO Common Pipeline Library
  *   Copyright (C) 2001-2005 European Southern Observatory
@@ -20,14 +20,17 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/12/13 12:58:16 $
- * $Revision: 1.22 $
- * $Name: cpl-6_1_1 $
+ * $Date: 2012/07/30 18:05:38 $
+ * $Revision: 1.23 $
+ * $Name: cpl-6_2 $
  */
 
 #undef CX_DISABLE_ASSERT
 #undef CX_LOG_DOMAIN
 
+#include "cpl_test.h"
+#include "cpl_fits.h"
+
 #include <stdio.h>
 #include <string.h>
 
@@ -37,7 +40,6 @@
 #include <cpl_msg.h>
 #include <cpl_memory.h>
 
-#include "cpl_test.h"
 #include "cpl_frameset_io.h"
 #include "cpl_image_io.h"
 
@@ -207,11 +209,7 @@ int main(void)
     */
     
     for (has_primary = 0; has_primary <= 1; has_primary++) {
-#ifdef CPL_IO_FITS_HAS_NO_FLUSH
-        const cxsize isave = CX_N_ELEMENTS(names);
-#else
         const cxsize isave = 1;
-#endif
         cxlong ii;
 
         /* The fits file contains three or four images */
@@ -248,7 +246,12 @@ int main(void)
             cpl_test_zero(system(cmd));
             cpl_free(cmd);
         }
-
+        if (has_primary) {
+            /* If the above cp'ed FITS files are cached by CPL,
+               then the cache is dirty, so refresh it */
+            if (cpl_fits_get_mode() == CPL_FITS_START_CACHING)
+                cpl_fits_set_mode(CPL_FITS_RESTART_CACHING);
+        }
         /* 
          * Test 2D: Test error-handling of cpl_imagelist_load_frameset()
          with invalid type.
