@@ -1,4 +1,4 @@
-/* $Id: cxtypes.h,v 1.3 2011/02/21 14:15:31 rpalsa Exp $
+/* $Id: cxtypes.h,v 1.4 2012/08/17 13:58:45 rpalsa Exp $
  *
  * This file is part of the ESO C Extension Library
  * Copyright (C) 2001-2011 European Southern Observatory
@@ -67,17 +67,19 @@ CX_BEGIN_DECLS
  * For completeness: Definitions for standard types
  */
 
-typedef char  cxchar;
-typedef short cxshort;
-typedef int   cxint;
-typedef long  cxlong;
-typedef cxint cxbool;
+typedef char      cxchar;
+typedef short     cxshort;
+typedef int       cxint;
+typedef long      cxlong;
+typedef long long cxllong;
+typedef cxint     cxbool;
 
-typedef unsigned char  cxuchar;
-typedef unsigned short cxushort;
-typedef unsigned int   cxuint;
-typedef unsigned long  cxulong;
-typedef cxuchar        cxbyte;
+typedef unsigned char      cxuchar;
+typedef unsigned short     cxushort;
+typedef unsigned int       cxuint;
+typedef unsigned long      cxulong;
+typedef unsigned long long cxullong;
+typedef cxuchar            cxbyte;
 
 typedef float  cxfloat;
 typedef double cxdouble;
